#define _SVID_SOURCE
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pthread.h>
#include "definition.h"
#include "graphique/interface.h"
#include "graphique/grille.h"
#include "liste.h"
#include "main.h"
#include "deplacement.h"


/**
* Interface du joueur de voiture
* initialise l'ensemble des outils utiles
* C'est ici que le joueur clique pour jouer
* @param argc
* @param argv
* @return EXIT_SUCCESS si tout c'est bien passe sinon EXIT_FAILURE
*/
int main(int argc, char *argv[])
{
	
	srand(time(NULL));
	if(argc != 3)
	{
		printf("./Client IP_serveur port\n");
		exit(-1);
	}
	
	int socket_data = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	int port_data = atoi(argv[2]);
	int taille[2];///////0 x , 1 y
	char * pos_actuelle;
	int fin_jeux = 0;
	int argent_total;
	MessageReseauCarte_t message;
	fenetre_t win_informations, win_grille, win_legende;
	grille_t grille;
	argument_t argumentthread;
	pthread_t thread_lecture;
	pthread_t thread_deplacement;
	position_t pos_joueur;
	
	struct sockaddr_in adresse_conf ;
	memset(&adresse_conf,0,sizeof(struct sockaddr_in));
	adresse_conf.sin_family = AF_INET;
	
	
	adresse_conf.sin_port = htons(port_data);
	inet_pton(AF_INET ,argv[1],&adresse_conf.sin_addr);


	port_data = initialise_connexion(adresse_conf);
	adresse_conf.sin_port = htons(port_data);
	printf("port data: %d\n",port_data);

	sleep(1);
	if(connect(socket_data,(struct sockaddr*)&adresse_conf,sizeof(struct sockaddr_in)) == -1)
	{
		fprintf(stderr, "Erreur connect : %s\n",strerror(errno));
		return -1;
	}
	
	enum type_joueur_e type = HELICO;
	
	if(write(socket_data,&type,sizeof(enum type_joueur_e)) == -1)
	{
		fprintf(stderr, "Erreur a l'envoie du type de client : %s\n",strerror(errno));
		return -1;
	}
	
	if(read(socket_data,&pos_joueur,sizeof(position_t)) == -1)
	{
		fprintf(stderr, "Erreur a la reception de la position joueur : %s\n",strerror(errno));
		return -1;
	}
	
	
	if(read(socket_data,&argent_total,sizeof(int)) == -1)
	{
		fprintf(stderr, "Erreur a la reception de la position joueur : %s\n",strerror(errno));
		return -1;
	}
	printf("argent total : %d \n",argent_total);
	
	if(read(socket_data,taille,2*sizeof(int)) == -1)
	{
		fprintf(stderr, "Erreur a la reception de la carte : %s\n",strerror(errno));
		return -1;
	}
	
	printf("construction de la carte \n");
	
	
	
	grille.largeur = taille[1];
	grille.hauteur = taille[0];
	grille.cases = malloc(sizeof(char)*grille.largeur * grille.hauteur);
	
	
	
	pos_actuelle = grille.cases;
	
	if(read(socket_data,&message,sizeof(MessageReseauCarte_t)) == -1)
	{
		fprintf(stderr, "Erreur a la reception de la carte : %s\n",strerror(errno));
		return -1;
	}
	
	memcpy ( pos_actuelle, message.carte, message.taille_utiliser_carte * sizeof(char));
	pos_actuelle += message.taille_utiliser_carte;
	
	while(message.paquet_total != message.paquet_no)
	{
		if(read(socket_data,&message,sizeof(MessageReseauCarte_t)) == -1)
		{
			fprintf(stderr, "Erreur a la reception de la carte : %s\n",strerror(errno));
			return -1;
		}
		memcpy ( pos_actuelle, message.carte, message.taille_utiliser_carte * sizeof(char));
		pos_actuelle += message.taille_utiliser_carte;
	}
	
	
	creation_interface(&grille, &win_informations, &win_legende, &win_grille);
	
	pthread_mutex_t *mutex_socket = malloc(sizeof(pthread_mutex_t));
	if (pthread_mutex_init(mutex_socket, NULL) != 0)
	{
		fprintf(stderr, "Erreur creation mutex\n");
	}
	
	pthread_mutex_t *mutex_Position_choisis = malloc(sizeof(pthread_mutex_t));
	if (pthread_mutex_init(mutex_Position_choisis, NULL) != 0)
	{
		fprintf(stderr, "Erreur creation mutex\n");
	}
	
	pthread_cond_t *reception_validation_deplacement_user = malloc(sizeof(pthread_cond_t));
	if (pthread_cond_init(reception_validation_deplacement_user, NULL) != 0)
	{
		fprintf(stderr, "Erreur creation variable condition\n");
	}
	
	
	
	fenetre_setpos(&win_grille, pos_joueur.y, pos_joueur.x);
	fenetre_affcar(&win_grille, 'H',8);
	
	position_t pos_dest_choisis;
	pos_dest_choisis.x = pos_joueur.x;
	pos_dest_choisis.y = pos_joueur.y;
	
	
	argumentthread.fd = socket_data;
	argumentthread.argent_restant = &argent_total;
	argumentthread.win_grille = &win_grille;
	argumentthread.win_info = &win_informations;
	argumentthread.grille = &grille;
	argumentthread.ma_position = &pos_joueur;
	argumentthread.pos_dest_choisis = &pos_dest_choisis;
	argumentthread.mutex_socket = mutex_socket;
	argumentthread.mutex_Position_choisis = mutex_Position_choisis;
	argumentthread.reception_validation_deplacement_user = reception_validation_deplacement_user;
	argumentthread.fin_jeux = &fin_jeux;
	
	// transformation de la socket en NON BLOCK 
	fcntl(socket_data, F_SETFL, fcntl(socket_data, F_GETFL, 0) | O_NONBLOCK);
	
	if(pthread_create(&(thread_lecture), NULL, routineLecture, (void *) &argumentthread) != 0)
	{
		printf("erreur create thread lecture\n");
		exit(0);
	}
	
	if(pthread_create(&(thread_deplacement), NULL, routineDeplacement, (void *) &argumentthread) != 0)
	{
		printf("erreur create thread deplacement\n");
		exit(0);
	}
	
	int ch,sourisx,sourisy,x,y;
	MessageReseauOrdre_t ordre;
	ordre.type = 3;
	
	/* Boucle principale */
	while((ch = getch()) != KEY_F(3)) 
	{
		switch(ch) 
		{
		case KEY_MOUSE:
			if(souris_getpos(&sourisx, &sourisy) == OK) 
			{
				/* Clic dans la fenetre de la grille */
				if(fenetre_getcoordonnees(&win_grille, sourisx, sourisy, &y, &x) == 1) 
				{ 
					pthread_mutex_lock (mutex_Position_choisis);
					pos_dest_choisis.x = x;
					pos_dest_choisis.y = y;
					pthread_cond_signal(reception_validation_deplacement_user);
					pthread_mutex_unlock (mutex_Position_choisis);
				}
			}
			break;
		case KEY_F(2):
			ordre.ordre = 2;
			pthread_mutex_lock (mutex_socket);
			if(write(socket_data,&ordre, sizeof(MessageReseauOrdre_t)) == -1)
			{
				fprintf(stderr,"Erreur dans l'arret du client\n");
			}

			pthread_mutex_unlock (mutex_socket);
			break;
		}
	}
	
	
	
	ordre.ordre = 1;
	pthread_mutex_lock (mutex_socket);
	if(write(socket_data,&ordre, sizeof(MessageReseauOrdre_t)) == -1)
	{
		fprintf(stderr,"Erreur dans l'arret du client\n");
	}
	pthread_mutex_unlock (mutex_socket); 
	sleep(2); // on attend un peut que l'ordre soit traite
	fin_jeux = 1; // on dit au thread quitte toi quand tu n'a plus de message dans la socket
	
	

	/* Fin de ncurses */
	fenetre_detruire(&win_informations);
	fenetre_detruire(&win_grille);
	ncurses_stopper();
	
	
	return 0;
}

/**
* Retourne le port sur lequel envoyer les donnees
* @param localaddr Structure de description de la socket
* @return Numero du port sinon EXIT_FAILURE (-1)
*/
int initialise_connexion(struct sockaddr_in localaddr)
{
	int port_config = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(port_config == -1)
	{
		fprintf(stderr, "Erreur lors de la creation de la socket pour le port de config : %s\n",strerror(errno));
		exit(EXIT_FAILURE);
	}
	printf("INTITIALISATION de la connexion\n");


	Commande_t ma_commande;
	ma_commande.type_client = HELICO;

	// on envoie la commande pour cree un port au switch
	sendto(port_config, &ma_commande, sizeof(Commande_t), 0,(struct sockaddr *)&localaddr, sizeof(struct sockaddr_in));

	printf("Commande envoyer\n");
	ReponseCommande_t var;

	if(recv(port_config, &var, sizeof(ReponseCommande_t), 0) == -1)
	{
		fprintf(stderr, "Erreur lors de la reception d'ordre pour le port de config : %s\n",strerror(errno));
		return -1;
	}
	close(port_config);

	if(var.statue == -1)
	{
		fprintf(stderr, "Le serveur de connexion a refuse la demande de connexion du client\n");
		exit(EXIT_FAILURE);
	}
	return var.port;
}

/**
* Creation de l'interface principale avec les trois fenetres.
* @param grille la grille de jeu
* @param win_informations la fenetre contenant les informations
* @param win_grille la fenetre contenant la grille de jeu
* @param win_outils la fenetre contenant les outils
*/
void creation_interface(grille_t *grille, fenetre_t *win_informations,fenetre_t * win_legende, fenetre_t *win_grille) {
	/* Initialisation de ncurses */
	ncurses_initialiser();
	ncurses_initcouleurs();
	ncurses_initsouris();
	curs_set(0);
	/* Verification des dimensions du terminal */
	if(ncurses_verifiedim(grille->largeur + 16,
				grille->hauteur + 10) == -1) {
		ncurses_stopper();
		fprintf(stderr, "\033[1m[ERREUR]\033[0m La taille du terminal n'est pas assez grande (%d, %d).\n", COLS, LINES);
		fprintf(stderr, "         Veuillez l'aggrandir et executez a nouveau l'application.\n");
		exit(EXIT_FAILURE);
	}

	fenetre_affch(NULL, "Client helico   --- Pressez F3 pour quitter le jeux -- F2 Pause\\play ----");

	/* Fenetre pour la grille */
	*win_grille = fenetre_creer(grille->hauteur + 2, grille->largeur + 2, 8, 0, "Ville", 1);
	grille_afficher_fenetre(win_grille, grille);

	/* Fenetre d'informations */
	*win_informations = fenetre_creer(7, COLS, 1, 0, "Informations", 0);
	fenetre_affch(win_informations, "Grille de %d par %d cases\n", grille->largeur, grille->hauteur);
	fenetre_maj(win_informations);

	*win_legende = fenetre_creer(grille->hauteur + 2, COLS - grille->largeur - 3, 8, grille->largeur + 3, "Legende", 1);
	fenetre_affcar(win_legende, CAR(MUR), COL(MUR));
	fenetre_affch(win_legende, " : Mur\n");
	fenetre_affcar(win_legende, CAR(BANQUE), COL(BANQUE));
	fenetre_affch(win_legende, " : Banque\n"); 
	fenetre_affcar(win_legende, CAR(PRISON), COL(PRISON));
	fenetre_affch(win_legende, " : Prison\n"); 
	fenetre_affcar(win_legende, CAR(REPERE), COL(REPERE));
	fenetre_affch(win_legende, " : Repere\n");
	fenetre_affcar(win_legende, CAR(PIETON), COL(PIETON));
	fenetre_affch(win_legende, " : Pieton\n"); 
	fenetre_affcar(win_legende, CAR(FOURGON), COL(FOURGON));
	fenetre_affch(win_legende, " : Fourgon\n"); 
	fenetre_affcar(win_legende, CAR(TERRORISTE), COL(TERRORISTE));
	fenetre_affch(win_legende, " : Terroriste\n"); 
	fenetre_affcar(win_legende, CAR(TERRORISTE_PRISON), COL(TERRORISTE_PRISON));
	fenetre_affch(win_legende, " : Terroriste en Prison\n"); 
	fenetre_affcar(win_legende, CAR(HELICO_DEF), COL(HELICO_DEF));
	fenetre_affch(win_legende, " : Helico\n"); 
	fenetre_affcar(win_legende, CAR(PATROUILLE), COL(PATROUILLE));
	fenetre_affch(win_legende, " : Patrouille\n"); 

	fenetre_maj(win_legende);


}

