#define _SVID_SOURCE
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <pthread.h>


#include "graphique/interface.h"
#include "graphique/grille.h"
#include "definition.h"
#include "liste.h"
#include "deplacement.h"


/**
* Thread de lecture des deplacements dans le jeu pour le joueur
* les deplacements peuvent aussi bien venir d'une Figurant que de l'autre Joueur
* cette routine recoit aussi les accuses de reception de le routineDeplacement
* @param arg Pointeur vers les arguments du thread 
*/
void* routineLecture(void *arg)
{
	
	argument_t *mes_arg = (argument_t *) arg;
	MessageReseauDeplacement_t message_reseau;
	MessageValidation_t message_validation;
	void *decalage_deplacement = (char *) &message_reseau + sizeof(long);
	void *decalage_validation = (char *) &message_validation + sizeof(long);
	list *listeFigurant = creerListeFigurant();
	int octet_lu;
	position_t autre_joueur;
	long type;

	while(1)
	{
		if(*(mes_arg->argent_restant) == 0)
		{
			fenetre_affch(mes_arg->win_info, "Partie perdu , les banques ont fait faillite, la ville est foutu !\n");
			fenetre_maj(mes_arg->win_info);
			close(mes_arg->fd);
			return NULL;
		}
		
		pthread_mutex_lock (mes_arg->mutex_socket);
		octet_lu = read(mes_arg->fd,&type, sizeof(long));
		
		
		if(octet_lu == -1 || octet_lu == 0)
		{
			if(EAGAIN == errno)
			{
				
			}
			else if(EAGAIN == errno && *(mes_arg->fin_jeux) == 1 )
			{
				return NULL;
			}
			else
			{
				printf("terminaison du port : le serveur est mort snif ...\n");
				close(mes_arg->fd);
				exit(EXIT_FAILURE); 
			}
		}
		else if(type == 1)
		{
			octet_lu = read(mes_arg->fd,decalage_deplacement, sizeof(MessageReseauDeplacement_t)-sizeof(long));
			if(octet_lu == -1 || octet_lu == 0)
			{
				if(octet_lu == 0)
				{
					printf("terminaison du port : le serveur est mort snif ...\n");
					close(mes_arg->fd);
					exit(EXIT_FAILURE); 
				}
				else if(EAGAIN != errno)
				{
					fprintf(stderr, "Erreur lors de la reception d'un message : %s\n",strerror(errno));
				}
			}
			else
			{
				if(message_reseau.joueur == 1)
				{
					if(autre_joueur.x != -1 && autre_joueur.y != -1)
					{
						fenetre_setpos(mes_arg->win_grille, autre_joueur.y, autre_joueur.x);
						cellule* repainfigurant = rechercherCoord(autre_joueur.x,autre_joueur.y, listeFigurant);
						if(mes_arg->ma_position->x == autre_joueur.x && mes_arg->ma_position->y == autre_joueur.y)
						{
							fenetre_affcar(mes_arg->win_grille, 'X',8);
						}
						else if(repainfigurant != NULL)
						{
							if(repainfigurant->type == 'P')
							{
								fenetre_affcar(mes_arg->win_grille, 'P',1);
							}
							else if(repainfigurant->type == 'T')
							{
								fenetre_affcar(mes_arg->win_grille, 'T',6);
							}
							else
							{
								fenetre_affcar(mes_arg->win_grille, 'U',7);
							}
						}
						else
						{
							fenetre_affcar(mes_arg->win_grille, ' ',1);
						}
						fenetre_maj(mes_arg->win_grille);
					}
					autre_joueur.x = message_reseau.position.x;
					autre_joueur.y = message_reseau.position.y;
					fenetre_setpos(mes_arg->win_grille, autre_joueur.y, autre_joueur.x);
					fenetre_affcar(mes_arg->win_grille, 'H',8);
					fenetre_maj(mes_arg->win_grille);
				}
				else
				{
					cellule* figurant = rechercher(message_reseau.figurantid, listeFigurant);
					if(figurant == NULL) // premier deplacement de se figurant on l'ajoute seulement au terrain 
					{
						figurant = malloc(sizeof(cellule));
						figurant->idFigurant = message_reseau.figurantid;
						figurant->x = message_reseau.position.x;
						figurant->y = message_reseau.position.y;
						figurant->type = ' ';
						figurant->bloquer = 0;
						insererCellule( figurant, listeFigurant);

					}
					else
					{
						// on met l'ancienne position a vide   
						int x = figurant->x, y = figurant->y;
						int ancienne_pos = (x * mes_arg->grille->largeur) + y;
						figurant->x = message_reseau.position.x;
						figurant->y = message_reseau.position.y;
						
						fenetre_setpos(mes_arg->win_grille, y, x);
						cellule* repainfigurant = rechercherCoord(x,y, listeFigurant);
						if(mes_arg->ma_position->x == x && mes_arg->ma_position->y == y)
						{
							fenetre_affcar(mes_arg->win_grille, 'X',8);
						}
						else if(repainfigurant != NULL)
						{
							if(repainfigurant->type == 'P')
							{
								fenetre_affcar(mes_arg->win_grille, 'P',1);
							}
							else if(repainfigurant->type == 'T')
							{
								fenetre_affcar(mes_arg->win_grille, 'T',6);
							}
							else
							{
								fenetre_affcar(mes_arg->win_grille, 'U',7);
							}
						}
						else
						{
							fenetre_affcar(mes_arg->win_grille, mes_arg->grille->cases[ancienne_pos],COL(TYPE(mes_arg->grille->cases[ancienne_pos])));
						}
						fenetre_maj(mes_arg->win_grille);
					}
					
					if(mes_arg->grille->cases[(message_reseau.position.x * mes_arg->grille->largeur) + message_reseau.position.y] == '#')
					{
						*(mes_arg->argent_restant) += message_reseau.argent;
						fenetre_affch(mes_arg->win_info, "Un Terroriste (%s) a été mis en prison [%d,%d] on recupere %dk euros\n",message_reseau.nom,message_reseau.position.x,message_reseau.position.y,message_reseau.argent,mes_arg->argent_restant);
						
						fenetre_maj(mes_arg->win_info);
						
						figurant->bloquer = 1;
					}
					
					if(figurant->bloquer == 1 && mes_arg->grille->cases[(message_reseau.position.x * mes_arg->grille->largeur) + message_reseau.position.y] != '#')
					{
						fenetre_affch(mes_arg->win_info, "Un Terroriste (%s) a été remis en liberte [%d,%d] :(\n",message_reseau.nom ,message_reseau.position.x,message_reseau.position.y);
						fenetre_maj(mes_arg->win_info);
						
						figurant->bloquer = 0;
					}


					fenetre_setpos(mes_arg->win_grille, message_reseau.position.y, message_reseau.position.x);
					if(Pieton == message_reseau.figurant)
					{
						fenetre_affcar(mes_arg->win_grille, 'P',1);
						if(figurant->type != 'P' && figurant->type != ' ')
						{
							fenetre_affch(mes_arg->win_info, "Un Terroriste (%s) a été blanchi [%d,%d] :(\n",message_reseau.nom,message_reseau.position.x,message_reseau.position.y);
							fenetre_maj(mes_arg->win_info);
						}
						figurant->type = 'P';
					}
					else if(Terroriste == message_reseau.figurant)
					{
						if(figurant->bloquer == 1)
						{
							fenetre_affcar(mes_arg->win_grille, 'T',9);
						}
						else
						{
							fenetre_affcar(mes_arg->win_grille, 'T',6);
						}

						if(figurant->type != 'T' && figurant->type != ' ')
						{
							*(mes_arg->argent_restant) -= message_reseau.argent;
							fenetre_affch(mes_arg->win_info, "Un Terroriste(%s) est apparu [%d,%d] ! vol de %dk euros a la banque  : %d\n",message_reseau.nom,message_reseau.position.x,message_reseau.position.y,message_reseau.argent , *(mes_arg->argent_restant) );
							fenetre_maj(mes_arg->win_info);
						}
						figurant->type = 'T';
					}
					else
					{
						fenetre_affcar(mes_arg->win_grille, 'U',7);
						figurant->type = 'U';
					}

					fenetre_setpos(mes_arg->win_grille, 80, 80);
					fenetre_maj(mes_arg->win_grille);

					// ON ENVOIE L'ACK
					MessageValidation_t validation;
					validation.type = 2;
					validation.figurantid = message_reseau.figurantid;
					validation.joueur = 0;
					validation.position = message_reseau.position;

					if(write(mes_arg->fd,&validation, sizeof(MessageValidation_t)) == -1)
					{
						fprintf(stderr,"Erreur dans l'envoie du message d'accuse reception de deplacement\n");
					}
				}
			}
		}
		else
		{
			octet_lu = read(mes_arg->fd,decalage_validation, sizeof(MessageValidation_t)-sizeof(long));
			if(octet_lu == -1 || octet_lu == 0)
			{
				if(octet_lu == 0)
				{
					printf("terminaison du port : le serveur est mort snif ...\n");
					close(mes_arg->fd);
					exit(EXIT_FAILURE); 
				}
				else if(EAGAIN != errno)
				{
					fprintf(stderr, "Erreur lors de la reception d'un message : %s\n",strerror(errno));
				}
			}
			else
			{
				int ancienne_pos = (mes_arg->ma_position->x * mes_arg->grille->largeur) + mes_arg->ma_position->y;
				fenetre_setpos(mes_arg->win_grille,mes_arg->ma_position->y , mes_arg->ma_position->x);
				fenetre_affcar(mes_arg->win_grille, mes_arg->grille->cases[ancienne_pos],COL(TYPE(mes_arg->grille->cases[ancienne_pos])));
				fenetre_maj(mes_arg->win_grille);
				
				fenetre_setpos(mes_arg->win_grille, message_validation.position.y, message_validation.position.x);
				fenetre_affcar(mes_arg->win_grille, 'X',8);
				fenetre_maj(mes_arg->win_grille);
				
				mes_arg->ma_position->x = message_validation.position.x;
				mes_arg->ma_position->y = message_validation.position.y;
				
				
				// EMITION DU REVEIL DU THREAD  DE LECTURE DEPLACEMENT  JOUEUR
				pthread_cond_signal(mes_arg->reception_validation_deplacement_user);
			}	
		}
		pthread_mutex_unlock (mes_arg->mutex_socket);	  
	}
	return NULL;
}


/** 
* Thread d'envoie d'ordre et de deplacement au serveur de connexion
* @param arg Pointeur vers les arguments du thread 
*/
void* routineDeplacement(void *arg)
{
	argument_t *mes_arg = (argument_t *) arg;
	int direction_x,direction_y;
	pthread_mutex_lock (mes_arg->mutex_socket);
	int attente_ack = 0;
	while(1)
	{
		if(*(mes_arg->argent_restant) == 0)
		{
			return NULL;
		}
		while ((mes_arg->pos_dest_choisis->x == mes_arg->ma_position->x && mes_arg->pos_dest_choisis->y == mes_arg->ma_position->y) ||attente_ack == 1)
		{
			pthread_cond_wait (mes_arg->reception_validation_deplacement_user, mes_arg->mutex_socket);
			attente_ack = 0;
		}
		// ON LIBERE LA MUTEX DE SOCKET CAR ON VA DORMIR SINON LE JOUEUR SE DEPLACERA TROP RAPIDEMENT
		pthread_mutex_unlock (mes_arg->mutex_socket); 

		pthread_mutex_lock (mes_arg->mutex_Position_choisis);
		direction_x = mes_arg->pos_dest_choisis->x - mes_arg->ma_position->x;
		direction_y = mes_arg->pos_dest_choisis->y - mes_arg->ma_position->y;
		pthread_mutex_unlock (mes_arg->mutex_Position_choisis);


		MessageReseauDeplacement_t message_reseau;
		message_reseau.type = 1;
		message_reseau.joueur = 1;
		message_reseau.position.x = mes_arg->ma_position->x;
		message_reseau.position.y = mes_arg->ma_position->y;


		if(direction_x != 0)
		{
			if(direction_x < 0)
			{
				message_reseau.position.x--;
			}
			else
			{
				message_reseau.position.x++;
			}
		}
		if(direction_y != 0)
		{
			if(direction_y < 0)
			{
				message_reseau.position.y--;
			}
			else
			{
				message_reseau.position.y++;
			}
		}



		usleep(300000);

		pthread_mutex_lock (mes_arg->mutex_socket);
		attente_ack = 1;
		if(write(mes_arg->fd,&message_reseau, sizeof(MessageReseauDeplacement_t)) == -1)
		{
			fprintf(stderr,"Erreur dans l'envoie du deplacement du joueur\n");
		}
	}
}