#include <stdlib.h>     /* Pour exit, EXIT_FAILURE, EXIT_SUCCESS */
#include <stdio.h>      /* Pour fprintf, printf */
#include <fcntl.h>      /* Pour open */
#include <unistd.h>     /* Pour read, write */
#include <sys/stat.h>   /* Pour S_IRUSR, S_IWUSR */

#include "interface.h"
#include "grille.h"

/**
 * Creation d'une nouvelle grille.
 * @param largeur la largeur de la grille
 * @param hauteur la hauteur de la grille
 * @return la grille creee
 **/
grille_t *grille_creer(int largeur, int hauteur) {
  grille_t *resultat = (grille_t*)malloc(sizeof(grille_t));
  int i;

  resultat->largeur = largeur;
  resultat->hauteur = hauteur;
  if((resultat->cases = (char*)malloc(sizeof(char) * largeur * hauteur)) == NULL) {
    perror("Erreur, grille_creer : probleme d'allocation ");
    exit(EXIT_FAILURE);
  }

  for(i = 0; i < largeur * hauteur; i++)
    resultat->cases[i] = CAR(VIDE);

  return resultat;
}

/**
 * Suppresion d'une grille.
 * @param grille la grille
 **/
void grille_supprimer(grille_t **grille) {
  free((*grille)->cases);
  free(*grille);
  *grille = NULL;
}

/**
 * Affichage d'une case de la grille dans une fenetre ncurses.
 * @param fenetre la fenetre
 * @param case le contenu de la case
 */
void grille_afficher_fenetre_case(fenetre_t *fenetre, char valcase) {
  int type = TYPE(valcase);
  
  fenetre_affcar(fenetre, valcase, COL(type));
}

/**
 * Affichage d'une grille dans une fenetre ncurses.
 * @param fenetre la fenetre dans laquelle afficher la grille
 * @param grille la grille a afficher
 **/
void grille_afficher_fenetre(fenetre_t *fenetre, grille_t *grille) {
  int i, j;

  for(i = 0; i < grille->hauteur; i++) {
    for(j = 0; j < grille->largeur; j++)
      grille_afficher_fenetre_case(fenetre, grille->cases[i * grille->largeur + j]);
  }
  fenetre_maj(fenetre);
}

/**
 * Affichage d'une grille.
 * @param grille la grille a afficher
 **/
void grille_afficher(grille_t *grille) {
  int i, j;

  printf("/");
  for(i = 0; i < grille->largeur; i++)
    printf("-");
  printf("\\\n");

  for(i = 0; i < grille->hauteur; i++) {
    printf("|");
    for(j = 0; j < grille->largeur; j++) {
      printf("%c", grille->cases[i * grille->largeur + j]);
    }
    printf("|\n");
  }
  
  printf("\\");
  for(i = 0; i < grille->largeur; i++)
    printf("-");
  printf("/\n");
}

/**
 * Sauvegarde d'une grille dans un fichier.
 * @param grille la grille a sauvegarder
 * @param nomFichier le nom du fichier
 **/
void grille_sauver(grille_t *grille, char *nomFichier) {
  int fd;

  if((fd = open(nomFichier, O_CREAT | O_TRUNC | O_WRONLY, S_IRUSR | S_IWUSR)) == -1) {
    perror("Erreur, grille_sauver : erreur lors de la creation du fichier ");
    exit(EXIT_FAILURE);
  }

  if(write(fd, grille, sizeof(int) * 2) == -1) {
    perror("Erreur, grille_sauver : erreur lors de l'ecriture dans le fichier (1) ");
    exit(EXIT_FAILURE);
  }
  if(write(fd, grille->cases, sizeof(char) * grille->largeur * grille->hauteur) == -1) {
    perror("Erreur, grille_sauver : erreur lors de l'ecriture dans le fichier (2) ");
    exit(EXIT_FAILURE);
  }

  if(close(fd) == -1) {
    perror("Erreur, grille_sauver : erreur lors de la fermeture du fichier ");
    exit(EXIT_FAILURE);
  }
}

/**
 * Chargement d'une grille depuis un fichier.
 * @param nomFichier le nom du fichier a ouvrir
 * @return la grille chargee
 **/
grille_t *grille_charger(char *nomFichier) {
  int fd, val[2];
  grille_t *resultat;

  if((fd = open(nomFichier, O_RDONLY)) == -1) {
    perror("Erreur, grille_charger : erreur lors de l'ouverture du fichier ");
    exit(EXIT_FAILURE);
  }

  if(read(fd, val, sizeof(int) * 2) == -1) {
    perror("Erreur, grille_charger : erreur lors de la lecture (1) ");
    exit(EXIT_FAILURE);
  }

  resultat = grille_creer(val[0], val[1]);  

  if(read(fd, resultat->cases, sizeof(char) * val[0] * val[1]) == -1) {
    perror("Erreur, grille_charger : erreur lors de la lecture (2) ");
    exit(EXIT_FAILURE);
  }  

  if(close(fd) == -1) {
    perror("Erreur, grille_charger : erreur lors de la fermeture du fichier ");
    exit(EXIT_FAILURE);
  }

  return resultat;
}
