#ifndef _GRILLE_
#define _GRILLE_

/**
 * Les differents types de cases.
 */
#define VIDE    0
#define MUR     1
#define BANQUE  2
#define PRISON  3
#define REPERE  4
#define PIETON 5
#define TERRORISTE 6
#define TERRORISTE_PRISON 10
#define FOURGON 7
#define HELICO_DEF 8
#define PATROUILLE 9


#define CAR_VIDE ' '
#define CAR_MUR  'X'
#define CAR_BANQUE '$'
#define CAR_PRISON '#'
#define CAR_REPERE 'R'

#define CAR_PIETON 'P'
#define CAR_TERRORISTE 'T'
#define CAR_FOURGON 'U'
#define CAR_HELICO 'H'
#define CAR_PATROUILLE 'X'

#define COL_VIDE   1
#define COL_MUR    5
#define COL_BANQUE 2
#define COL_PRISON 4
#define COL_REPERE 3

#define COL_PIETON 1
#define COL_TERRORISTE 6
#define COL_FOURGON 7
#define COL_HELICO 8
#define COL_PATROUILLE 8
#define COL_TERRORISTE_PRISON 9


#define TYPE(valcase) (						\
		       (valcase) == CAR_MUR ? MUR :		\
		       (valcase) == CAR_BANQUE ? BANQUE :	\
		       (valcase) == CAR_PRISON ? PRISON :	\
		       (valcase) == CAR_REPERE ? REPERE :	\
		       VIDE)

#define CAR(tcase) (					\
		    (tcase)== MUR ? CAR_MUR :		\
		    (tcase)== BANQUE ? CAR_BANQUE :	\
		    (tcase)== PRISON ? CAR_PRISON :	\
		    (tcase)== REPERE ? CAR_REPERE :	\
		    (tcase)== PIETON ? CAR_PIETON :	\
		    (tcase)== TERRORISTE ? CAR_TERRORISTE :	\
		    (tcase)== FOURGON ? CAR_FOURGON :	\
		    (tcase)== HELICO_DEF ? CAR_HELICO :	\
		    (tcase)== PATROUILLE ? CAR_PATROUILLE :	\
		    (tcase)== TERRORISTE_PRISON ? CAR_TERRORISTE :	\
		    CAR_VIDE)

#define COL(tcase) (					\
		    (tcase)== MUR ? COL_MUR :		\
		    (tcase)== BANQUE ? COL_BANQUE :	\
		    (tcase)== PRISON ? COL_PRISON :	\
		    (tcase)== REPERE ? COL_REPERE :	\
		    (tcase)== PIETON ? COL_PIETON :	\
		    (tcase)== TERRORISTE ? COL_TERRORISTE :	\
		    (tcase)== FOURGON ? COL_FOURGON :	\
		    (tcase)== HELICO_DEF ? COL_HELICO :	\
		    (tcase)== PATROUILLE ? COL_PATROUILLE :	\
		    (tcase)== TERRORISTE_PRISON ? COL_TERRORISTE_PRISON :	\
		    COL_VIDE)

/**
 * Structure de la grille.
 */
typedef struct {
  int largeur;
  int hauteur;
  char *cases;
} grille_t;

/**
 * Creation d'une nouvelle grille.
 * @param largeur la largeur de la grille
 * @param hauteur la hauteur de la grille
 * @return la grille creee
 **/
grille_t *grille_creer(int largeur, int hauteur);

/**
 * Suppresion d'une grille.
 * @param grille la grille
 **/
void grille_supprimer(grille_t **grille);

/**
 * Affichage d'une case de la grille dans une fenetre ncurses.
 * @param fenetre la fenetre
 * @param valcase le contenu de la case
 */
void grille_afficher_fenetre_case(fenetre_t *fenetre, char valcase);

/**
 * Affichage d'une grille dans une fenetre ncurses.
 * @param fenetre la fenetre dans laquelle afficher la grille
 * @param grille la grille a afficher
 **/
void grille_afficher_fenetre(fenetre_t *xfenetre, grille_t *grille);

/**
 * Affichage d'une grille.
 * @param grille la grille a afficher
 **/
void grille_afficher(grille_t *grille);

/**
 * Sauvegarde d'une grille dans un fichier.
 * @param grille la grille a sauvegarder
 * @param nomFichier le nom du fichier
 **/
void grille_sauver(grille_t *grille, char *nomFichier);

/**
 * Chargement d'une grille depuis un fichier.
 * @param nomFichier le nom du fichier a ouvrir
 * @return la grille chargee
 **/
grille_t *grille_charger(char *nomFichier);

#endif
