#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "liste.h"

/**
* Fonction qui creer une liste chainee 
* alloue le premier element
*/
list* creerListeFigurant()
{
	list* l = (list*)malloc(sizeof(list));
	l->debut = NULL;
	return l;
}

/**
* Fonction qui supprime le premier element de la liste
* @param l Liste en question
*/
void removeFirstElement(list* l)
{
	if(l != NULL && l->debut != NULL)
	{
		l->debut = l->debut->next;
		free(l->debut->last);
		l->debut->last = NULL;
	}
}

/**
* Fonction tous les elements de la liste
* @param l Liste en question
*/
void removeAllElement(list* l)
{
	if(l != NULL)
	{
		while(l->debut != NULL)
		{
			removeFirstElement(l);
		}
	}
}


/**
* Fonction qui supprime le dernier element de la liste
* @param l Liste en question
*/
void removeLastElement(list* l)
{
	cellule* var = getLastElement(l);
	if(var->last != NULL)
	{
		var->last->next = NULL;
	}
	free(var);
}


/**
* Fonction qui retourne le premier element de la liste
* @param l Liste en question
* @return Premiere cellule de la liste
*/
cellule* getFirstElement(list* l)
{
	return l->debut;
}
/**
* Fonction qui retourne le dernier element de la liste
* @param l Liste en question
* @return Derniere cellule de la liste
*/
cellule* getLastElement(list* l)
{
	if(l != NULL && l->debut != NULL)
	{
		cellule* precedent = l->debut;
		while(precedent != NULL)
		{
			if(precedent->next == NULL)
			{
				return precedent->next;
			}
			precedent = precedent->next;
		}	
	}
	return NULL;
}

/**
* Fonction qui qui ajoute un element de la liste
* @param l Liste en question
* @return Cellule cree
*/
cellule* creerCellule(int idFigurant,int x,int y)
{
	cellule* c = (cellule*)malloc(sizeof(cellule));
	c->idFigurant = idFigurant;
	c->x = x;
	c->y = y;
	c->last = NULL;
	c->next = NULL;
	return c;
}

/**
* Fonction detruit chaque element de la liste et la liste
* @param l Liste en question
*/
void detruireListeFigurant(list* l)
{
	if(l != NULL && l->debut != NULL)
	{
		cellule *precedent = l->debut;
		cellule *suivant = l->debut->next;
		while(precedent != NULL)
		{
			free(precedent);
			precedent = suivant;
			if(precedent != NULL)
			suivant = precedent->next;
		}	
	}
	free(l);
	
}

/**
* Fonction detruit chaque element de la liste et la liste
* @param l Liste en question
*/
void insererCoord(int x,int y, list* l)
{
	cellule* c = creerCellule(0,x,y);
	if(c != NULL)
	{
		if(l == NULL || l->debut == NULL)
		{
			l->debut = c;
		}
		else
		{
			cellule *suivant = l->debut;
			l->debut = c;
			c->next = suivant;
			suivant->last = c;
		}
	}
}

/**
* Fonction qui permet d'inserer une nouvelle cellule dans une liste
* @param c Celllule a inserer 
* @param l Liste dans laquelle inserer la cellule
*/
void insererCellule(cellule* c, list* l)
{
	if(c != NULL)
	{
		if(l == NULL || l->debut == NULL)
		{
			l->debut = c;
		}
		else
		{
			cellule *suivant = l->debut;
			l->debut = c;
			c->next = suivant;
			suivant->last = c;
		}
	}
}

/**
* Fonction qui permet de rechercher une cellule par un identifiant 
* @param idFigurant Id que l'on cherche dans la cellule
* @param l Liste en question
* @return Cellule trouvee sinon null
*/
cellule* rechercher(int idFigurant, list* l)
{
	if(l != NULL && l->debut != NULL)
	{
		cellule* precedent = l->debut;
		while(precedent != NULL)
		{
			if(precedent->idFigurant == idFigurant)
			{
				return precedent;
			}
			precedent = precedent->next;
		}
		return NULL;	
	}
	return NULL;
}

/**
* Fonction recherche dans chaque element de la liste si il y en a un qui contient les valeurs
* @param x Valeur x de la cellule a trouver
* @param y Valeur y de la cellule a trouver
* @param l Liste en question
* @return Cellule trouvee ou null
*/
cellule* rechercherCoord(int x,int y, list* l)
{
	if(l != NULL && l->debut != NULL)
	{
		cellule* precedent = l->debut;
		while(precedent != NULL)
		{
			if(precedent->x == x && precedent->y == y)
			{
				return precedent;
			}
			precedent = precedent->next;
		}
		return NULL;	
	}
	return NULL;
}