#define _SVID_SOURCE
#define _GNU_SOURCE
#include <arpa/inet.h>
#include "fils.h"
#include "definition.h"
void *memoire_partager = NULL;
int socketdesc = -1;
int noPort;
int deplacement_x = -1;
int deplacement_y = -1;
int deplacement_joueur_en_cour = 0;
int bloquer = 0;

/**
* Quitte proprement le programme en tuant tous ces fils 
* et desaloueant les pointeurs seulement sur SIGINT ou SIGQUIT
* si SIGUSR1 remet le thread en action 
* si SIGUSR2 met le thread en pause 		-> gestion play/pause du jeu
* @param num_signal Numero du signal recu
*/
void gestionnaire_signaux_fils(int num_signal)
{
	if(num_signal == SIGQUIT || num_signal == SIGINT)
	{
		config_memoire * memoire = (config_memoire * )memoire_partager;
		close(socketdesc);
		memoire->joueur[noPort].pid = 0;
		exit(1);	
	}
	if(num_signal == SIGUSR1)
	{
		bloquer = 0;
	}
	if(num_signal == SIGUSR2)
	{
		bloquer = 1;
	}
}

/**
* Routine qui va simuler le comportement d'un port de switch il se contente de recevoir et d'envoyer les requetes 
* qu'il recoit de son client le joueur (il est lie a un et un seul joueur)
* @param port Port sur lequel le thread va creer sa socket d'ecoute tcp
* @param confIPC Pointeur vers la structures qui contient les adresses des outils ipc du jeu
*/
void fils(int port,int casememoire,config_outil_IPC_t * confIPC)
{
	noPort = casememoire;
	
	struct sigaction ordre;
	sigemptyset(&(ordre.sa_mask));
	ordre.sa_handler = gestionnaire_signaux_fils;
	ordre.sa_flags = 0;
	
	sigaction(SIGQUIT,&ordre,NULL);
	sigaction(SIGINT,&ordre,NULL);
	sigaction(SIGUSR1,&ordre,NULL);
	sigaction(SIGUSR2,&ordre,NULL);
	
	
	

	if((memoire_partager = shmat(confIPC->seg_memoire_id, NULL,0)) == (void *)-1)
	{
		printf("Une erreur est survenue lors de l'attachement de la memoire partager sur le port %d \n",casememoire);
		exit(EXIT_FAILURE);
	}
	
	if((confIPC->tab_semaphore_id = semget(confIPC->clefTS, 0, 0)) == -1)
	{
		printf("Une erreur est survenue lors de la creation de l'array de semaphore\n");
		exit(EXIT_FAILURE);
	}


	int ma_socket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
	if(ma_socket == -1)
	{
		fprintf(stderr, "Erreur lors de la creation de la socket pour le port %d : %s\n",casememoire,strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	
	
	
	config_memoire* memoire = (config_memoire *)memoire_partager;
	
	
	struct sockaddr_in localaddr ;
	memset(&localaddr,0,sizeof(struct sockaddr_in));
	localaddr.sin_family = AF_INET; 
	localaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	localaddr.sin_port = htons(port);



	if(bind(ma_socket,(struct sockaddr*)&localaddr,sizeof(struct sockaddr_in)) == -1)
	{
		fprintf(stderr, "Erreur lors du Bind pour le port %d : %s\n",casememoire,strerror(errno));
		close(ma_socket);
		memoire->joueur[noPort].pid = 0;
		exit(EXIT_FAILURE);
	}

	if (listen(ma_socket,2) == -1)
	{
		fprintf(stderr, "Erreur lors de la mise en ecoute pour le port %d: %s\n",casememoire,strerror(errno));
		close(ma_socket);
		memoire->joueur[noPort].pid = 0;
		exit(EXIT_FAILURE);
	}


	struct sockaddr_in adresse_source;
	socklen_t adresse_length = sizeof(struct sockaddr_in);
	int connexionfd=accept(ma_socket,(struct sockaddr*)&adresse_source,&adresse_length);
	socketdesc = connexionfd;
	close(ma_socket);
	
	memoire->joueur[noPort].IPdest = adresse_source;
	
	// on receptionne le type de client
	
	if(read(connexionfd,&(memoire->joueur[noPort].type),sizeof(enum type_joueur_e)) == -1)
	{
		fprintf(stderr, "Erreur a la reception de la carte : %s\n",strerror(errno));
		close(connexionfd);
		memoire->joueur[noPort].pid = 0;
		exit(EXIT_FAILURE);
	}
	
	memoire->joueur[noPort].pos = getPositionLibre();
	
	while(write(connexionfd,&(memoire->joueur[noPort].pos), sizeof(position_t)) == -1)
	{
		if(EINTR != errno)
		{
			fprintf(stderr,"Erreur dans l'envoie du message de validation de deplacement\n");
			close(connexionfd);
			memoire->joueur[noPort].pid = 0;
			exit(-1);
		}
	}
	// ENVOIE DE L'ARGENT TOTAL DU JEU
	while(write(connexionfd,&(memoire->argent_restant), sizeof(int)) == -1)
	{
		if(EINTR != errno)
		{
			fprintf(stderr,"Erreur dans l'envoie du message de validation de deplacement\n");
			close(connexionfd);
			memoire->joueur[noPort].pid = 0;
			exit(-1);
		}
	}
	
	// ON ENVOIE LA CARTE AU JOUEUR
	printf("envoie carte\n");
	envoieCarte(connexionfd);
	
	
	// on connecte le joueur au jeux 
	memoire->joueur[noPort].pid = getpid();
	
	// transformation de la socket en NON BLOCK 
	fcntl(connexionfd, F_SETFL, fcntl(connexionfd, F_GETFL, 0) | O_NONBLOCK);

	
	
	//// ENVOIE POSITION a l'autre AUTRE JOUEUR //// 
	
	MessageReseauDeplacement_t deplacement;    
	deplacement.joueur = 1; // savoir si le message vient du joueur
	
	if(noPort == 0 && memoire->joueur[1].pid != 0)
	{
		deplacement.position.x = memoire->joueur[0].pos.x;
		deplacement.position.y = memoire->joueur[0].pos.y;
		deplacement.type = memoire->joueur[1].pid;
		while(msgsnd(confIPC->file_message_id, &deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
		{
			if(EINTR != errno)
			{
				fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de conection.\n");
				close(connexionfd);
				memoire->joueur[noPort].pid = 0;
				exit(-1);
			}
		}
	}
	else if(noPort == 1 && memoire->joueur[0].pid != 0)
	{
		deplacement.position.x = memoire->joueur[1].pos.x;
		deplacement.position.y = memoire->joueur[1].pos.y;
		deplacement.type = memoire->joueur[0].pid;
		while(msgsnd(confIPC->file_message_id, &deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
		{
			if(EINTR != errno)
			{
				fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de conection.\n");
				close(connexionfd);
				memoire->joueur[noPort].pid = 0;
				exit(-1);
			}
		}
	}
	
	long type;
	int octet_lu;
	//////////////////: BOUCLE PRINCIPALE //////////////
	while(1)
	{
		
		//REGARDE QUEL TYPE DE PAQUET ON A RECU
		
		traiterDeplacement(connexionfd,confIPC);
		
		octet_lu = read(connexionfd,&type, sizeof(long));
		if(octet_lu == -1 || octet_lu == 0)
		{
			if(EAGAIN == errno || ENOMSG == errno)
			{
			}
			else
			{
				fprintf(stderr, "Erreur lors de la reception d'un message sur la socket dans la boucle principale : %s\n",strerror(errno));
				close(connexionfd);
				memoire->joueur[noPort].pid = 0;
				exit(0);
			}
		}
		else
		{
			if(type == 1) // On a recu un deplacement du client
			{
				receptionDeplacement(connexionfd,confIPC);
			}
			if(type == 2)
			{
				receptionAccuseReception(connexionfd,confIPC);
			}
			if(type == 3)
			{
				receptionPause(connexionfd);
			}
		}
		if(bloquer == 0)
		{
			envoieDeplacementFigurant(connexionfd,confIPC);
		}

		
	}
	/*
	la connexion doit renvoyer un message au thread pour qu'il debloque la semaphore bloque
	la connexion attend le ACK du client avant de renvoyer un message au thread
	
	Le thread figurant bloque la semaphore fait et envoie le deplacement dans la file de message    FAIT
	le thread figurant est bloque sur la fille de message en attendant le ACK de la connexion     FAIT
	
	la connexion est NON BLOCK avec la socket et la FILE De message      -FAIT
	la connexion check les ACK du client et les renvoi au thread quand il y en a   -FAIT
	la connection check les messages de la file de message et les envoie au client quand il y en a. -FAIT
	
	Le client envoie des delacements a la connexion et est bloque en attendant un ACK
	Le thread reception Recoit les donnees de connexion , affiche le deplacement , renvoie un ACK a la connexion
	
Thread Reception ---					---- Thread figurant
	<!-- Client ---|-- Connexion ----- File message  ---- Thread figurant
							---- Thread figurant
	*/
}


/**
* Fonction qui va préparer la mise en pause du jeu a la reception de l'ordre de pause
* @param fd Descripteur de fichier de la socket tcp
*/
void receptionPause(int fd)
{
	MessageReseauOrdre_t *message_ordre = malloc(sizeof(MessageReseauOrdre_t));
	void *decalage = (char *) message_ordre + sizeof(long);
	config_memoire * memoire = (config_memoire * )memoire_partager ;
	
	
	int octet_lu = read(fd,decalage, sizeof(MessageReseauOrdre_t)-sizeof(long));
	
	while(octet_lu == -1 || octet_lu == 0)
	{
		if(EAGAIN == errno)
		{
			return;
		}
		else if(EINTR == errno)
		{
			octet_lu = read(fd,decalage, sizeof(MessageReseauOrdre_t)-sizeof(long));
		}
		else
		{
			if(octet_lu == 0)
			{
				fprintf(stderr, "Le client c'est barre en vacances : deconnexion\n");
			}
			else
			{
				fprintf(stderr, "Erreur lors de la reception d'un message sur la socket dans receptionPause() : %s\n",strerror(errno));
			}
			close(fd);
			memoire->joueur[noPort].pid = 0;
			exit(0);
		}
	}
	
	if(message_ordre->ordre == 1)
	{
		memoire->joueur[noPort].pid = 0;
	}
	else
	{
		
		if(bloquer == 1)
		{
			bloquer = 0;
			if(noPort == 0 && memoire->joueur[1].pid != 0)
			{
				kill( memoire->joueur[1].pid,SIGUSR1);
			}
			else if(noPort == 1 && memoire->joueur[0].pid != 0)
			{
				kill(memoire->joueur[0].pid ,SIGUSR1);
			}
		}
		else
		{
			bloquer = 1;
			if(noPort == 0 && memoire->joueur[1].pid != 0)
			{
				kill(memoire->joueur[1].pid,SIGUSR2 );
			}
			else if(noPort == 1 && memoire->joueur[0].pid != 0)
			{
				kill(memoire->joueur[0].pid,SIGUSR2 );
			}
		}
		//pour mettre en pause le jeu
	}
	free(message_ordre);
}

/**
* Fonction qui va envoyer un accuse de reception au serveur (a travers la file de message) pour signifier que le client a bien recu 
* et traiter le deplacement qu'on lui a envoye plus tot et renvoye son accuse de reception
* @param fd Descripteur de fichier de la socket tcp
* @param confIPC Pointeur vers la structures qui contient les adresses des outils ipc du jeu
*/
void receptionAccuseReception(int fd,config_outil_IPC_t * confIPC)
{
	MessageValidation_t *message_validation = malloc(sizeof(MessageValidation_t));
	void *decalage = (char *) message_validation + sizeof(long);
	config_memoire * memoire = (config_memoire * )memoire_partager ;
	int octet_lu = read(fd,decalage, sizeof(MessageValidation_t)-sizeof(long));
	
	while(octet_lu == -1 || octet_lu == 0)
	{
		if(EAGAIN == errno)
		{
			return;
		}
		else if(EINTR == errno)
		{
			octet_lu = read(fd,decalage, sizeof(MessageValidation_t)-sizeof(long));
		}
		else
		{
			if(octet_lu == 0)
			{
				fprintf(stderr, "Le client c'est barre en vacances : deconnexion\n");
			}
			else
			{
				fprintf(stderr, "Erreur lors de la reception d'un message sur la socket dans receptionAccuseReception()  : %s\n",strerror(errno));
			}
			close(fd);
			memoire->joueur[noPort].pid = 0;
			exit(0);
		}
	}
	
	message_validation->type = message_validation->figurantid;
	
	while(msgsnd(confIPC->file_message_id, message_validation,sizeof(MessageValidation_t)-sizeof(long),0) == -1)
	{
		if(errno != EINTR)
		{
			fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur dans la file IPC receptionAccuseReception()\n");
			return;
		}
	}
	free(message_validation);
}

/**
* Fonction qui va traiter le deplacement que le ServeurFils vient de recevoir
* Pour cela il transfere la structure de deplacement dans la file de message pour qu'elle soit traite par le serveur
* et enregistre sur le segment memoire 
* Une fois cela fait, il envoie un message au joueur pour valider son deplacement
* @param fd Descripteur de fichier de la socket tcp
* @param confIPC Pointeur vers la structure qui contient les adresses des outils ipc du jeu
*/
void traiterDeplacement(int fd,config_outil_IPC_t * confIPC)
{
	if(deplacement_joueur_en_cour == 1 && bloquer == 0)
	{
		config_memoire * memoire = (config_memoire * )memoire_partager ;
		if(verrouiller_deverrouiller_semaphore(memoire,confIPC,1,deplacement_x, deplacement_y) == -1)
		{
			return;
		}
		//////////TEST SI LA CASE EST LIBRE/////////:
		if(memoire->joueur[noPort].type == HELICO || caseLibre(memoire,Pieton,deplacement_x,deplacement_y) == 1)
		{
			/////////////DEPLACEMNT ////////////
			memoire->joueur[noPort].pos.x = deplacement_x;
			memoire->joueur[noPort].pos.y = deplacement_y;

			////////////VERIF CAMBRIOLEUR/////////
			if(memoire->joueur[noPort].type != HELICO) // il y a que si on a un vehicule qu'on peut arreter le terroriste
			{
				char *carte = (void *)memoire_partager + sizeof(config_memoire);
				Figurant_t * fourgon = (void *)carte + (sizeof(char)*(memoire->taille_carte_x*memoire->taille_carte_y));
				Figurant_t * pieton = (void *)fourgon + (sizeof(Figurant_t)*memoire->nb_figurant_fourgon);
				int i;
				for(i = 0; i < memoire->nb_figurant_pieton; i++)
				{
					if(pieton[i].type == Terroriste && 
							pieton[i].pos.x == memoire->joueur[noPort].pos.x && 
							pieton[i].pos.y == memoire->joueur[noPort].pos.y )
					{
						pieton[i].bloquer = 1;
					}
				}
			}
			
			
			MessageReseauDeplacement_t deplacement;
			deplacement.joueur = 1; // savoir si le message vient du joueur
			deplacement.position.x = deplacement_x;
			deplacement.position.y = deplacement_y;
			if(noPort == 0 && memoire->joueur[1].pid != 0)
			{
				deplacement.type = memoire->joueur[1].pid;
				while(msgsnd(confIPC->file_message_id, &deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
				{
					if(EINTR != errno)
					{
						fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de conection.\n");
						close(fd);
						memoire->joueur[noPort].pid = 0;
						exit(-1);
					}
				}
			}
			else if(noPort == 1 && memoire->joueur[0].pid != 0)
			{
				deplacement.type = memoire->joueur[0].pid;
				while(msgsnd(confIPC->file_message_id, &deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
				{
					if(EINTR != errno)
					{
						fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de conection.\n");
						close(fd);
						memoire->joueur[noPort].pid = 0;
						exit(-1);
					}
				}
			}
			

			MessageValidation_t message;
			message.type = 2;
			message.figurantid = -1;
			message.joueur = 1;
			message.position.x = deplacement_x;
			message.position.y = deplacement_y;
			
			while(write(fd,&message, sizeof(MessageValidation_t)) == -1)
			{
				if(EINTR != errno)
				{
					fprintf(stderr,"Erreur dans l'envoie du message de validation de deplacement\n");
					close(fd);
					memoire->joueur[noPort].pid = 0;
					exit(-1);
				}    
			}
		}
		deplacement_x = -1;
		deplacement_y = -1;
		deplacement_joueur_en_cour = 0;

		////////FIN VERROUILLAGE MEMOIRE////////////
		verrouiller_deverrouiller_semaphore(memoire,confIPC,-1,deplacement_x, deplacement_y);
	}
}

/**
* Fonction qui va determiner de qui vient le deplacement et agir en concequence
* @param fd Descripteur de fichier de la socket tcp
* @param confIPC Pointeur vers la structures qui contient les adresses des outils ipc du jeu
*/
void receptionDeplacement(int fd,config_outil_IPC_t * confIPC)
{
	MessageReseauDeplacement_t *message_reseau = malloc(sizeof(MessageReseauDeplacement_t));
	void *decalage = (char *) message_reseau + sizeof(long);
	
	
	int octet_lu = read(fd,decalage, sizeof(MessageReseauDeplacement_t)-sizeof(long));
	config_memoire * memoire = (config_memoire * )memoire_partager ;
	
	while(octet_lu == -1 || octet_lu == 0)
	{
		if(EAGAIN == errno)
		{
			return; 
		}
		else if(EINTR == errno)
		{
			octet_lu = read(fd,decalage, sizeof(MessageValidation_t)-sizeof(long));
		}
		else
		{
			if(octet_lu == 0)
			{
				fprintf(stderr, "Le client c'est bare en vacances : deconnexion\n");
			}
			else
			{
				fprintf(stderr, "Erreur lors de la reception d'un message sur la socket dans receptionDeplacement() : %s\n",strerror(errno));
			}
			fprintf(stderr, "Erreur lors de la reception d'un message : %s\n",strerror(errno));
			close(fd);
			memoire->joueur[noPort].pid = 0;
			exit(0);
		}
	}
	
	printf("deplacement recu\n");
	if(bloquer == 1)
	{
		return;
	}
	
	
	int x = memoire->joueur[noPort].pos.x, y = memoire->joueur[noPort].pos.y;
	
	deplacement_x = x;
	deplacement_y = y;
	deplacement_joueur_en_cour = 1;
	//////////////VERROUILLAGE DE LA MEMOIRE ////////
	
	if(verrouiller_deverrouiller_semaphore(memoire,confIPC,1,x, y) == -1) // si on arrive pas a verrouiller la semaphore 
	{
		return;
	}
	
	
	//////////TEST SI LA CASE EST LIBRE/////////:
	if(caseLibre(memoire,memoire->joueur[noPort].type,message_reseau->position.x,message_reseau->position.y) == 1)
	{
		/////////////DEPLACEMNT ////////////
		memoire->joueur[noPort].pos.x = message_reseau->position.x;
		memoire->joueur[noPort].pos.y = message_reseau->position.y;
		////////////VERIF CAMBRIOLEUR/////////
		
		if(memoire->joueur[noPort].type != HELICO) // il y a que si on a un vehicule qu'on peut arreter le terroriste
		{
			char *carte = (void *)memoire_partager + sizeof(config_memoire);
			Figurant_t * fourgon = (void *)carte + (sizeof(char)*(memoire->taille_carte_x*memoire->taille_carte_y));
			Figurant_t * pieton = (void *)fourgon + (sizeof(Figurant_t)*memoire->nb_figurant_fourgon);
			int i;
			for(i = 0; i < memoire->nb_figurant_pieton; i++)
			{	
				if(pieton[i].type == Terroriste && 
						pieton[i].pos.x == memoire->joueur[noPort].pos.x && 
						pieton[i].pos.y == memoire->joueur[noPort].pos.y )
				{
					pieton[i].bloquer = 1;
				}
			}		  
		}
		
		MessageReseauDeplacement_t deplacement;
		deplacement.joueur = 1; // savoir si le message vient du joueur
		deplacement.position.x = deplacement_x;
		deplacement.position.y = deplacement_y;
		if(noPort == 0 && memoire->joueur[1].pid != 0)
		{
			deplacement.type = memoire->joueur[1].pid;
			while(msgsnd(confIPC->file_message_id, &deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
			{
				if(EINTR != errno)
				{
					fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de connexion.\n");
					close(fd);
					memoire->joueur[noPort].pid = 0;
					exit(-1);
				}
			}
		}
		else if(noPort == 1 && memoire->joueur[0].pid != 0)
		{
			deplacement.type = memoire->joueur[0].pid;
			while(msgsnd(confIPC->file_message_id, &deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
			{
				if(EINTR != errno)
				{
					fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de connexion.\n");
					close(fd);
					memoire->joueur[noPort].pid = 0;
					exit(-1);
				}
			}
		}
		
		MessageValidation_t message;
		message.type = 2;
		message.figurantid = -1;
		message.joueur = 1;
		message.position.x = message_reseau->position.x;
		message.position.y = message_reseau->position.y;
		
		while(write(fd,&message, sizeof(MessageValidation_t)) == -1)
		{
			if(EINTR != errno)
			{
				fprintf(stderr,"Erreur dans l'envoie du message de validation de deplacement\n");
				close(fd);
				memoire->joueur[noPort].pid = 0;
				exit(-1);
			}
		}
	}
	
	deplacement_x = -1;
	deplacement_y = -1;
	deplacement_joueur_en_cour = 0;

	////////FIN VERROUILLAGE MEMOIRE////////////
	verrouiller_deverrouiller_semaphore(memoire,confIPC,-1,x, y);
	free(message_reseau);
}


/**
* Fonction qui permet d'envoyer un deplacement de Figurant au joueur 
* @param fd Descripteur de ficheir de la socket tcp
* @param confIPC Pointeur vers les adresses de outils ICP
*/
void envoieDeplacementFigurant(int fd,config_outil_IPC_t * confIPC)
{
	MessageReseauDeplacement_t message_reseau;
	config_memoire * memoire = (config_memoire * )memoire_partager ;
	while(msgrcv(confIPC->file_message_id, &message_reseau,sizeof(MessageReseauDeplacement_t)-sizeof(long),getpid(),IPC_NOWAIT) == -1)
	{
		if(errno == ENOMSG)
		{
			return;
		}
		else if(errno != EINTR)
		{
			fprintf(stderr, "Erreur lors de la reception d'un message reseau de la file de message sur le port %d.\n",noPort);
			close(fd);
			memoire->joueur[noPort].pid = 0;
			exit(-1);
		}
	}
	
	// pour que le client reconnaisse un message de deplacement
	message_reseau.type = 1;
	
	
	while(write(fd,&message_reseau, sizeof(MessageReseauDeplacement_t)) == -1)
	{
		if(EINTR != errno)
		{
			fprintf(stderr,"Erreur dans l'envoie du message de deplacement de figurant port n° %d\n",noPort);
			memoire->joueur[noPort].pid = 0;
			close(fd);
			exit(-1);
		} 
	}

}

/**
* Fonction qui envoie la carte du segment memoire dans la socket tcp
* @param fd Descripteur de fichier de la socket tcp 
*/
void envoieCarte(int fd)
{
	config_memoire * memoire = (config_memoire * )memoire_partager ;
	
	unsigned int nbmessage = (memoire->taille_carte_x * memoire->taille_carte_y)/500;
	if((memoire->taille_carte_x * memoire->taille_carte_y)%500 > 0)
	{
		nbmessage++;
	}
	MessageReseauCarte_t message;
	message.paquet_total = nbmessage;
	

	char *carte = memoire_partager + sizeof(config_memoire);
	unsigned int i = 1;
	unsigned int taille_carte = memoire->taille_carte_x * memoire->taille_carte_y;
	
	
	int taille_carte_tab[2];
	taille_carte_tab[0] = memoire->taille_carte_x;
	taille_carte_tab[1] = memoire->taille_carte_y;
	
	if(write(fd,taille_carte_tab, sizeof(int) * 2) == -1)
	{
		fprintf(stderr,"Erreur dans l'envoie de la carte sur le joueur n° %d\n",noPort);
		close(fd);
		memoire->joueur[noPort].pid = 0;
		exit(0);
	}

	
	while(taille_carte != 0)
	{
		if(taille_carte > 500)
		{
			message.paquet_no = i;
			message.taille_utiliser_carte = 500;
			memcpy ( message.carte, carte, 500*sizeof(char));
			taille_carte -= 500;
			carte += 500;
			if(write(fd,&message, sizeof(MessageReseauCarte_t)) == -1)
			{
				fprintf(stderr,"Erreur dans l'envoie de la carte sur le joueur n° %d\n",noPort);
				close(fd);
				memoire->joueur[noPort].pid = 0;
				exit(0);
			}
			i++;
		}
		else
		{
			message.paquet_no = i;
			message.taille_utiliser_carte = taille_carte;
			memcpy ( message.carte, carte, taille_carte*sizeof(char));
			taille_carte -= taille_carte;
			if(write(fd,&message, sizeof(MessageReseauCarte_t)) == -1)
			{
				fprintf(stderr,"Erreur dans l'envoie de la carte sur le joueur n° %d\n",noPort);
				close(fd);
				memoire->joueur[noPort].pid = 0;
				exit(0);
			}
		}
	}
}

/**
* Fonction qui dertermine si la case est libre, ou occupee par un Figurant ou si c'est un batiment
* @param configuration_jeu Pointeur vers le debut de la structure de configuration du segment memoire
* @param type Type du Figurant (pas les memes droits d esuperposition)
* @param x Position x a determiner
* @param y Position y a determiner
* @return 0 si la case n'est pas libre 1 elle ne l'est pas 
*/
int caseLibre(config_memoire * configuration_jeu,enum type_joueur_e type,int x,int y)
{

	
	int position = (x * configuration_jeu->taille_carte_y) + y;
	char * carte  = (void * )configuration_jeu + sizeof(config_memoire);
	
	if(x >= configuration_jeu->taille_carte_x || x < 0 || y >= configuration_jeu->taille_carte_y || y < 0)
	{
		return 0;
	}
	
	if(type == HELICO) 
	{
		return 1;
	}
	

	if(carte[position] == 'X' || carte[position] == '$' || carte[position] == '#' || carte[position] == 'R')
	{
		return 0;
	}
	
	return 1;


}
/**
* Fonction utilitaire qui devrrouille/deverrouille une ou des zones de la carte
* en fonction des coordonnees du Figurant
* @param configuration_jeu Pointeur vers le debut de la configuration du segment mémoire
* @param confIPC Pointeur sur la structure des adresses des outils ICP du jeu
* @param operation Operation qui l'on souhaire faire sur les semaphores P=1 V=-1
* @param x Position x du Figurant
* @param y Position y du Figurant
* @return 0 si cela s'est bien passe sinon -1
*/
int verrouiller_deverrouiller_semaphore(config_memoire * configuration_jeu,config_outil_IPC_t * confIPC,int operation,int x, int y)
{
	if(operation < 0 )
	operation = -1;
	else
	operation = 1;
	
	int zone_y = (configuration_jeu->taille_carte_y / 4);
	
	struct sembuf op[5];
	// des puis-je pour 

	op[0].sem_op = operation;
	op[0].sem_flg = IPC_NOWAIT;
	op[0].sem_num = 0;
	op[1].sem_op = operation;
	op[1].sem_flg = IPC_NOWAIT;
	op[1].sem_num = 0;
	op[2].sem_op = operation;
	op[2].sem_flg = IPC_NOWAIT;
	op[2].sem_num = 0;
	op[3].sem_op = operation;
	op[3].sem_flg = IPC_NOWAIT;
	op[3].sem_num = 0;
	op[4].sem_op = operation;
	op[4].sem_flg = IPC_NOWAIT;
	op[4].sem_num = 0;
	
	int zone_x_sem = (x / 4); 
	int zone_y_sem = (y / 4);
	
	int sem_base = zone_x_sem * zone_y + zone_y_sem;
	
	op[0].sem_num = sem_base;
	
	//0 bas, 1 haut, 2 guauche, 3 droite
	
	int i = 1;
	
	// si on est pas a la limite en bas
	if((x + 1) < configuration_jeu->taille_carte_x && ((((x+1) / 4) * zone_y) + zone_y_sem) != sem_base)
	{
		op[i].sem_num = (((x+1) / 4) * zone_y) + zone_y_sem;
		i++;
	}
	
	if((x - 1) >= 0 && (((((x-1) / 4) * zone_y) + zone_y_sem) != sem_base))
	{
		op[i].sem_num = (((x-1) / 4) * zone_y) + zone_y_sem;
		i++;
	}
	
	if((y - 1) >= 0 && ((zone_x_sem * zone_y) + ((y-1) / 4)) != sem_base)
	{
		op[i].sem_num = (zone_x_sem * zone_y) + ((y-1) / 4);
		i++;
	}
	if((y + 1) < configuration_jeu->taille_carte_y && ((zone_x_sem * zone_y) + ((y+1) / 4)) != sem_base)
	{
		op[i].sem_num = (zone_x_sem * zone_y) + ((y+1) / 4);
		i++;
	}

	while(semop(confIPC->tab_semaphore_id, op, i) == -1) 
	{
		if(EINTR != errno)
		{
			return -1;
		}
	}
	return 0;

}

/**
* Fonction qui determine une position libre pour un Figurant qui n'en a pas 
* Appelle au debut du jeu
* @return Coordonnees de la position trouve
*/
position_t getPositionLibre()
{
	config_memoire * memoire = (config_memoire * )memoire_partager ;
	int position_trouver = 0,x,y;
	position_t var;

	while(position_trouver == 0)
	{
		x = rand() % memoire->taille_carte_x;
		y = rand() % memoire->taille_carte_y;

		if(caseLibre(memoire,VOITURE,x,y) == 1)
		{
			position_trouver = 1;
			var.x = x;
			var.y = y;   
		}
	}
	return var;
}

