#define _SVID_SOURCE
#define _GNU_SOURCE
#define _POSIX_SOURCE

#include <arpa/inet.h>
#include "main.h"
#include "fils.h"

config_outil_IPC_t * conf_outil_IPC = NULL;
void *memoire_partager_pere = NULL;


/**
* Quitte proprement le programme en tuant tous ces fils 
* et desaloueant les pointeurs seulement sur SIGINT ou SIGQUIT
* @param num_signal Numero du signal recu
*/
void gestionnaire_signaux(int num_signal)
{
	if(num_signal == SIGQUIT || num_signal == SIGINT)
	{
		free(conf_outil_IPC);
		if(memoire_partager_pere != NULL)
		{
			int i;
			config_memoire * memoire = (config_memoire *) memoire_partager_pere;
			for(i = 0; i < 2; i++)
			{
				if(memoire->joueur[i].pid != 0)
				{
					kill(memoire->joueur[i].pid,SIGQUIT);
				}
			}
		}
		// on declenche la destruction de tout
		exit(EXIT_SUCCESS);
	}
}


/**
* Processus d'administration des ports joueur
* un joueur se logue -> le processus donne acces ou pas a un port qu'il aura cree
* dans un processus fils 
* @param arc 
* @param argv[]
* @return EXIT_SUCCESS si tout c'est bien passe sinon EXIT_FAILURE
*/
int main(int argc, char* argv[])
{
	if(argc != 3)
	{
		printf("./Connexion port FILEMESSID\n");
		exit(-1);
	}

	conf_outil_IPC = malloc(sizeof(config_outil_IPC_t));


	// definition du signal
	struct sigaction ordre;
	sigemptyset(&(ordre.sa_mask));

	ordre.sa_handler = gestionnaire_signaux;
	ordre.sa_flags = 0;

	sigaction(SIGQUIT,&ordre,NULL);
	sigaction(SIGINT,&ordre,NULL);



	conf_outil_IPC->clefFM = atoi(argv[2]);

	//creation de la file de message
	conf_outil_IPC->file_message_id = -1;
	conf_outil_IPC->seg_memoire_id = -1;
	conf_outil_IPC->tab_semaphore_id = -1;
	

	if((conf_outil_IPC->file_message_id = msgget(conf_outil_IPC->clefFM, 0)) == -1)
	{
		printf("Une erreur est survenue lors de la creation de la file de message : %s\n", strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	printf("On recup la config IPC \n");
	// ON VA RECUPERER LES AUTRES CLEF IPC via a une demande par la file de message
	RecupererConfig(conf_outil_IPC);
	printf("fin \n");
	//configuration du segment de memoire 

	if((conf_outil_IPC->seg_memoire_id = shmget(conf_outil_IPC->clefMP, 0 ,0)) == -1)
	{
		
		printf("Une erreur est survenue lors de la creation de la memoire partager\n");
		msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		exit(EXIT_FAILURE);	
	}



	//attachement du segment de memoire partagee
	if((memoire_partager_pere = shmat(conf_outil_IPC->seg_memoire_id, NULL,0)) == (void *)-1)
	{
		printf("Une erreur est survenue lors de l'attachement de la memoire partager\n");
		msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
		exit(EXIT_FAILURE);
	}

	//creation d'un pointeur vers la memoire partagee et mise a 0 de toute la memoire
	config_memoire * memoire = (config_memoire *) memoire_partager_pere;
	
	

	int port_config = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
	if(port_config == -1)
	{
		fprintf(stderr, "Erreur lors de la creation de la socket pour le port de config : %s\n",strerror(errno));
		msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
		exit(EXIT_FAILURE);
	}


	struct sockaddr_in localaddr ;
	memset(&localaddr,0,sizeof(struct sockaddr_in));
	localaddr.sin_family = AF_INET; 
	localaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	localaddr.sin_port = htons(atoi(argv[1]));


	if(bind(port_config,(struct sockaddr*)&localaddr,sizeof(struct sockaddr_in)) == -1)
	{
		fprintf(stderr, "Erreur lors du Bind pour le port de config : %s\n",strerror(errno));
		msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
		close(port_config);
		exit(EXIT_FAILURE);
	}

	printf("fin de l'optention des info\n");
	memoire->pid_gestionnaire_connection_pere = getpid();

	while(1)
	{
		sleep(1);
		receptionnerCommande(port_config,memoire);
	}
}


/**
* Fonction qui creer un nouveau port d'ecoute tcp dans un processus fils pour le client joueur
* @param memoire Pointeur vers le debut de la configuration du segment memoire
* @param typeclient Type de client que l'on doit creer
* @return Numero du port cree 
*/
int creePort(config_memoire * memoire,enum type_joueur_e typeclient)
{
	////////////////VERIFICATION DE DISPONIBILITE DES PORTS

	int i;
	int portlibre = -1;
	int case_a_utiliser = 0;
	for(i = 0; i < 2; i++)
	{
		if(memoire->joueur[i].pid == 0)
		{
			case_a_utiliser = i;
			i = 2;
			portlibre = 1;
			
		}
		else
		{
			if(typeclient == memoire->joueur[i].type)
			{
				return -1;
			}
		}
	}
	if(portlibre == -1)
	{
		return -1;
	}


	//////////////// CREATION D'UN PORT
	int port = selectPort();

	
	if(fork() == 0)
	{

		fils(port,case_a_utiliser,conf_outil_IPC);
		exit(0);
	}
	return port;
}

/**
* Fonction utilitaire qui cherche un port d'ecoute libre sur mon systeme et le retourne 
* @return Numero du port trouve
*/
int selectPort()
{
	struct sockaddr_in client;         
	int sock = socket(AF_INET, SOCK_STREAM, 0);
	if(sock == -1)
	{
		fprintf(stderr, "Erreur lors de la creation de la socket pour le test de port: %s\n",strerror(errno));
	}
	client.sin_addr.s_addr = htonl(INADDR_ANY);
	client.sin_family = AF_INET;
	
	
	int port = rand()%(30000-20000) + 20000;
	
	
	client.sin_port = htons(port);  
	
	while(connect(sock, (struct sockaddr *) &client,sizeof(client)) != -1)
	{
		close(sock);
		sock = socket(AF_INET, SOCK_STREAM, 0);
		port = rand()%(30000-20000) + 20000;
		client.sin_port = htons(port);
	}
	
	close(sock);
	return port;
}

/**
* On recoit une demande de login dans le jeu 
* on cree un nouveau port pour les communications entre le joueur et le serveur
* on envoie un accuse de reception 
* @param memoire Pointeur vers le debut de la configuration memoire du segment memoire
*/
void receptionnerCommande(int socket,config_memoire * memoire)
{
	printf("Attente d'ordre\n");

	Commande_t ma_commande;
	struct sockaddr_in adresse_source;
	socklen_t adresse_length = sizeof(struct sockaddr);;

	errno = 0;
	if(recvfrom(socket, &ma_commande, sizeof(Commande_t), 0,(struct sockaddr*)&adresse_source, &adresse_length) == -1)
	{
		fprintf(stderr, "Erreur lors de la reception d'ordre pour le port de config : %s\n",strerror(errno));
		return;
	}
	
	printf("creation du port en cour\n");
	int port = creePort(memoire,ma_commande.type_client);
	printf("port %d \n",port);
	reponseCommandeCreePort(socket,adresse_source, port );
}

/**
* Envoie un accuse de reception pour dire que le port est ouvert a la connexion tcp
* le joueur est admi dans le jeu
* @param socket Descipteur de fichier de la socket 
* @param adresse_source Description de la socket 
* @param port Numero du port de la connexion
*/
void reponseCommandeCreePort(int socket,struct sockaddr_in adresse_source, int port )
{
	ReponseCommande_t var;
	if(port == -1)
	{
		var.statue = -1;
	}
	else
	{
		var.statue = 1;
	}
	var.port = port;
	sendto(socket, &var, sizeof(ReponseCommande_t), 0,&adresse_source, sizeof(struct sockaddr_in));
}

/**
*	Fonction qui recupere et charge en memoire la configuration du jeu
* @param conf_IPC Pointeur vers la structure que l'on va remplir des adresses des outils IPC
*/
void RecupererConfig(config_outil_IPC_t *conf_IPC)
{
	demande_config_t ma_demande;
	ma_demande.pid = getpid();
	ma_demande.type = 1;

	// on envoie la demande de config
	if(msgsnd(conf_IPC->file_message_id, &ma_demande,sizeof(demande_config_t)-sizeof(long),0) == -1)
	{
		fprintf(stderr, "Erreur lors de l'envoie de la config a quelqu'un.\n");
	}

	config_outil_IPC_t var;

	if(msgrcv(conf_IPC->file_message_id, &var,sizeof(config_outil_IPC_t)-sizeof(long),getpid(),0) == -1)
	{
		fprintf(stderr, "Erreur lors de la reception d'un deplacement.\n");
		exit(EXIT_FAILURE);
	}

	conf_IPC->clefMP = var.clefMP;
	conf_IPC->clefTS = var.clefTS;
}


