#define _SVID_SOURCE
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <netinet/in.h>
#include "definition.h"
#include "actionthread.h"

/**
* Fonction qui permet de bloquer le pieton 
* C'est a dire qu'il s'est fait prendre par un joueur
* @param id_propre Id unique du Figurant pour recevoir son accuse de reception
* @param noInMemory Place du pieton dans le tableau de Figurant du semgment memoire
* @param configuration Pointeur vers la structure de configuration dans le segment memoire
* @param deplacement Pointeur du deplacement a realiser
*/
void blocagePieton(int id_propre, int noInMemory,config_memoire * configuration, MessageReseauDeplacement_t *deplacement)
{
	Figurant_t * fourgon = (void *)configuration + sizeof(config_memoire) 
	+ (sizeof(char)*(configuration->taille_carte_x*configuration->taille_carte_y));
	Figurant_t * pieton = (void *)fourgon + (sizeof(Figurant_t)*configuration->nb_figurant_fourgon);
	///////////PERMET DE BLOQUER UN FOURGON OU UN PIETON on a juste a lock le mutex :D

	if(pieton[noInMemory].bloquer == 1)
	{
		// ENVOIE D'UN DEPLACEMENT DANS UNE PRISON
		// on rend l'argent derobe :)
		pthread_mutex_lock (lockargent);
		configuration->argent_restant += deplacement->argent;
		configuration->argent_sauver += deplacement->argent;
		configuration->nb_terroriste_arreter++;
		pthread_mutex_unlock (lockargent);
		
		
		position_t coordprison = getCoordPrison(configuration,pieton[noInMemory].pos.x, pieton[noInMemory].pos.y);
		deplacement->position.x = coordprison.x;
		deplacement->position.y = coordprison.y;
		
		////////////////////////////////ENVOI DU DEPLACEMENT///////////////////////////////////
		envoieDeplacement(id_propre,configuration,deplacement);
		
		
		pieton[noInMemory].pos.y = coordprison.y;
		pieton[noInMemory].pos.x = coordprison.x;
		pieton[noInMemory].type = Pieton;
		deplacement->figurant = Pieton;
		deplacement->argent = 0;
		
		int temps_liberation = 10;
		
		// MISE EN PRISON PENDANT 10s : VARIABLE DE CONDITION LE BLOCAGE
		while (pieton[noInMemory].bloquer == 1 && temps_liberation != 0) 
		{
			printf("thread bloquer liberation dans : %d\n",temps_liberation);
			sleep(1);
			temps_liberation--;
		}
		pieton[noInMemory].bloquer = 0;
		
	}
}

/**
* Fonction qui permet de bloquer le fourgon
* C'est a dire qu'il est en train de se faire cambrioler 
* @param noInMemory Place du fourgon dans le tableau de Figurant du segment memoire
* @param configuration Pointeur vers le début de la structure de configuration du segment memoire
* @return 0 si le fourgon n'est pas bloque sinon 1
*/
int blocageFourgon(int noInMemory,config_memoire * configuration)
{
	Figurant_t * fourgon = (void *)configuration + sizeof(config_memoire) 
	+ (sizeof(char)*(configuration->taille_carte_x*configuration->taille_carte_y));
	///////////PERMET DE BLOQUER UN FOURGON OU UN PIETON on a juste a lock le mutex :D

	if(fourgon[noInMemory].bloquer == 1)
	{
		// MISE EN VARIBLE DE CONDITION LE BLOCAGE //////////// A VERIF VEROUILLIAGE SEM
		verrouiller_deverrouiller_semaphore(configuration,1,fourgon[noInMemory].pos.x, fourgon[noInMemory].pos.y); // on deverrouille le semaphore sinon un pan de la carte va s'arreter
		while (fourgon[noInMemory].bloquer == 1) 
		{
			sleep(1);
		}
		printf("deblocage fouron\n");
		return 1;
		
	}
	return 0;
}

/**
* Fonction qui envoie un deplacement recu dans la file de message
* au deux joueurs et attend leur accuse de reception respectif
* @param id_propre Id unique du Figurant pour recevoir son accuse de reception
* @param configuration Pointeur vers la structure de configuration dans le segment memoire
* @param deplacement Pointeur du deplacement a realiser
*/
void envoieDeplacement(int id_propre,config_memoire * configuration,MessageReseauDeplacement_t *deplacement)
{
	int client1_envoyer = 0,client2_envoyer = 0;
	////////////////////////////////ENVOI DU DEPLACEMENT///////////////////////////////////
	if(configuration->joueur[0].pid != 0)
	{
		//printf("envoie deplacement de %d\n",id_propre);
		deplacement->type = configuration->joueur[0].pid;
		pthread_mutex_lock(lockOutilsIPCFM);
		if(msgsnd(conf_outil_IPC->file_message_id, deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
		{
			fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de connexion.\n");
		}
		client1_envoyer = 1;
		pthread_mutex_unlock(lockOutilsIPCFM);
	}
	if(configuration->joueur[1].pid != 0)
	{
		//printf("envoie deplacement de %d\n",id_propre);
		deplacement->type = configuration->joueur[1].pid;
		pthread_mutex_lock(lockOutilsIPCFM);
		if(msgsnd(conf_outil_IPC->file_message_id, deplacement,sizeof(MessageReseauDeplacement_t)-sizeof(long),0) == -1)
		{
			fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de connexion.\n");
		}
		client2_envoyer = 1;
		pthread_mutex_unlock(lockOutilsIPCFM);
	}
	/////////////////////////////RECEPTION DE L'ACUSE //////////////////////////////////////
	MessageValidation_t var;
	if(configuration->joueur[0].pid != 0 || client1_envoyer == 1)
	{
		//printf("1 : attente d'un message a destination de %d \n",id_propre);
		pthread_mutex_lock(lockOutilsIPCFM);
		if(msgrcv(conf_outil_IPC->file_message_id, &var,sizeof(MessageValidation_t)-sizeof(long),id_propre,0) == -1)
		{
			fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de connexion\n");
		}
		pthread_mutex_unlock(lockOutilsIPCFM);
		//printf(" 1 :deblocage deplacement %d\n",id_propre);
	}
	if(configuration->joueur[1].pid != 0 || client2_envoyer == 1)
	{
		//printf("2 : attente d'un message a destination de %d \n",id_propre);
		pthread_mutex_lock(lockOutilsIPCFM);
		if(msgrcv(conf_outil_IPC->file_message_id, &var,sizeof(MessageValidation_t)-sizeof(long),id_propre,0) == -1)
		{
			fprintf(stderr, "Erreur lors de l'envoie du deplacement vers le serveur de connexion.\n");
		}
		pthread_mutex_unlock(lockOutilsIPCFM);
		//printf("2 : deblocage deplacement %d\n",id_propre);
	}
}

/**
* Fonction qui creer un deplacement aleatoirement et l'envoie dans la file de message
* @param id_propre Id unique du Figurant pour recevoir son accuse de reception
* @param configuration Pointeur vers la structure de configuration dans le segment memoire
* @param deplacement Pointeur du deplacement a realiser
* @param p_x Pointeur d'origine x du deplacement
* @param p_y Pointeur d'origine y du deplacement
*/
void deplacementFigurant(int id_propre,config_memoire * configuration, MessageReseauDeplacement_t *deplacement,int *p_x,int *p_y)
{
	//0 droite, 1 haut, 2 guauche , 3 bas
	int deplacement_rand = rand() % 4;
	//printf("coordonnees : x : %d y :%d deplacement : %d\n",*p_x,*p_y,deplacement);
	
	if(configuration->joueur[0].pid != 0 || configuration->joueur[1].pid != 0  )
	{
		//printf("Semaphore verrouiller %d\n",id_propre);
		int x = *p_x;
		int y = *p_y;
		switch(deplacement_rand)
		{
		case 0:
			
			
			if(caseLibre(configuration,deplacement->figurant,x,y-1))
			{
				
				deplacement->position.x = *p_x;
				deplacement->position.y = *p_y - 1;
			}

			break;
		case 1:
			
			if(caseLibre(configuration,deplacement->figurant,x+1,y))
			{
				
				deplacement->position.x = *p_x + 1;
				deplacement->position.y = *p_y;
			}
			
			break;
			
		case 2:
			
			if(caseLibre(configuration,deplacement->figurant,x,y+1))
			{
				deplacement->position.x = *p_x;
				deplacement->position.y = *p_y + 1;
			}
			
			break;
			
		case 3:
			if(caseLibre(configuration,deplacement->figurant,x-1,y))
			{
				deplacement->position.x = *p_x - 1;
				deplacement->position.y = *p_y;
			}
			
			break;

		}
		
		envoieDeplacement(id_propre,configuration,deplacement);
		
		*p_x = deplacement->position.x;
		*p_y = deplacement->position.y;
		//printf(" Deverrouillage semaphore id %d\n",id_propre);
	}   
}


/**
* Fonction qui donne les coordonnees de la prison la plus proche par rapport a un Figurant
* @param configuration_jeu Pointeur vers le debut de la structure de configuration du segment memoire
* @param x Coordonnee x du Figurant
* @param y Coordonnee y du Figurant 
* @return Coordonnees de la prison la plus proche
*/
position_t getCoordPrison(config_memoire * configuration_jeu,int x,int y)
{
	char * carte  = (void * )configuration_jeu + sizeof(config_memoire);
	
	int i,x_prison,y_prison;
	int taille_carte  = configuration_jeu->taille_carte_y * configuration_jeu->taille_carte_x;
	position_t prison;

	
	int distance_min = -1;
	int distance_x = -1;
	int distance_y = -1;
	
	for(i = 0; i < taille_carte ; i++)
	{
		if(carte[i] == '#')
		{
			x_prison = i / configuration_jeu->taille_carte_y;
			y_prison = i % configuration_jeu->taille_carte_y;
			
			distance_x = x_prison - x;
			distance_y = y_prison - y;
			
			if(distance_x < 0)
			distance_x *= -1;
			
			if(distance_y < 0)
			distance_y *= -1;
			
			if(distance_min == -1 || (distance_x + distance_y) < distance_min)
			{
				prison.x = x_prison;
				prison.y = y_prison;
				distance_min = distance_x + distance_y;
			}
		}
	}
	
	return prison;
	
}

/**
* Fonction qui va simuler ou non un cambriolage de banque ou de fourgon si elle se trouve proche du Figurant
* @param configuration_jeu Pointeur vers le debut de la structure de configuration du segment memoire
* @param x Coordonnee x du Pieton
* @param y Coordonnee y du Pieton 
* @return 1 si la banque ou le fourgon a ete cambriole sinon 0
*/
int cambriolerBanque(config_memoire * configuration_jeu,int x,int y)
{
	char *carte = (void *)configuration_jeu + sizeof(config_memoire);
	Figurant_t * fourgon = (void *)carte + (sizeof(char)*(configuration_jeu->taille_carte_x*configuration_jeu->taille_carte_y));
	if((x - 1)  >=  0)
	{
		int pos = ((x-1) * configuration_jeu->taille_carte_y) + y;
		if(carte[pos] == '$')
		{
			if( configuration_jeu->joueur[0].pos.x != (x-1) || configuration_jeu->joueur[0].pos.y != y)
			{
				if( configuration_jeu->joueur[1].pos.x != (x-1) || configuration_jeu->joueur[1].pos.y != y)
				{
					if((rand() % 100) < 20) // 20% de chance de cambrioler
					{
						return 1;
					}
				}
			}
			
		}
	}
	
	if((x + 1)  <  configuration_jeu->taille_carte_x)
	{
		int pos = ((x+1) * configuration_jeu->taille_carte_y) + y;
		if(carte[pos] == '$')
		{
			if(configuration_jeu->joueur[0].pos.x != (x+1) || configuration_jeu->joueur[0].pos.y != y) // on peut cambrioler !
			{
				if(configuration_jeu->joueur[1].pos.x != (x+1) || configuration_jeu->joueur[1].pos.y != y)
				{
					if((rand() % 100) < 20) // 20% de chance de cambrioler
					{
						return 1;
					}
				}
			}
		}
	}
	
	if((y + 1)  <  configuration_jeu->taille_carte_y)
	{
		int pos = (x * configuration_jeu->taille_carte_y) + y + 1;
		if(carte[pos] == '$')
		{
			if(configuration_jeu->joueur[0].pos.x != x || configuration_jeu->joueur[0].pos.y != (y+1)) // on peut cambrioler !
			{
				if(configuration_jeu->joueur[1].pos.x != x || configuration_jeu->joueur[1].pos.y != (y+1))
				{
					if((rand() % 100) < 20) // 20% de chance de cambrioler
					{
						return 1;
					}
				}
			}
		}
	}
	
	if((y - 1)  >= 0)
	{
		int pos = (x * configuration_jeu->taille_carte_y) + y - 1;
		if(carte[pos] == '$')
		{
			if(configuration_jeu->joueur[0].pos.x != x || configuration_jeu->joueur[0].pos.y != (y-1)) // on peut cambrioler !
			{
				if(configuration_jeu->joueur[1].pos.x != x || configuration_jeu->joueur[1].pos.y != (y-1))
				{
					if((rand() % 100) < 20) // 20% de chance de cambrioler
					{
						return 1;
					}
				}
			}
		}
	}
	
	int i;
	for(i = 0; i < configuration_jeu->nb_figurant_fourgon ; i++)
	{
		if(fourgon[i].pos.x == x && fourgon[i].pos.y == y) // les fourgons ne peuvent se marcher dessus
		{
			if((rand() % 100) < 20) // 20% de chance de cambrioler
			{
				return 1;
			}
		}
	}
	return 0;
}

/**
* Fonction qui simule la remise du butin quand on capture un Terroriste
* @param configuration_jeu Pointeur vers le debut de la structure de configuration du segment memoire
* @param x Coordonnee x du Pieton
* @param y Coordonnee y du Pieton 
* @return 1 si le cambrioleur a reussit a aller jusqu'a son repere sinon 0
*/
int remiseDuButin(config_memoire * configuration_jeu,int x,int y) // appelle que si on est un terroriste
{
	char *carte = (void *)configuration_jeu + sizeof(config_memoire);
	if((x - 1)  >=  0)
	{
		int pos = ((x-1) * configuration_jeu->taille_carte_y) + y;
		if(carte[pos] == 'R')
		{
			return 1;
		}
	}
	
	if((x + 1)  <  configuration_jeu->taille_carte_x)
	{
		int pos = ((x+1) * configuration_jeu->taille_carte_y) + y;
		if(carte[pos] == 'R')
		{
			return 1;
		}
	}
	
	if((y + 1)  <  configuration_jeu->taille_carte_y)
	{
		int pos = (x * configuration_jeu->taille_carte_y) + y + 1;
		if(carte[pos] == 'R')
		{
			return 1;
		}
	}
	
	if((y - 1)  >= 0)
	{
		int pos = (x * configuration_jeu->taille_carte_y) + y - 1;
		if(carte[pos] == 'R')
		{
			return 1;
		}
	}
	return 0;
}

/**
* Fonction qui tente de liberer un Terroriste en prison et passe son type a Pieton quand un pieton libre passe a cote 
* de la prison a cote de sa position (20% de chance que cela marche)
* @param configuration_jeu Pointeur vers le debut de la structure de configuration du segment memoire
* @param x Coordonnee x du Pieton
* @param y Coordonnee y du Pieton 
*/
void libererPrisonnier(config_memoire * configuration_jeu,int x,int y) // debloque les threads bloques si le pieton fait son action
{
	
	char *carte = (void *)configuration_jeu + sizeof(config_memoire);
	Figurant_t * fourgon = (void *)carte + (sizeof(char)*(configuration_jeu->taille_carte_x*configuration_jeu->taille_carte_y));
	Figurant_t * pieton = (void *)fourgon + (sizeof(Figurant_t)*configuration_jeu->nb_figurant_fourgon);
	int i ;
	
	if((x - 1)  >=  0)
	{
		int pos = ((x-1) * configuration_jeu->taille_carte_y) + y;
		if(carte[pos] == '#')
		{
			if(configuration_jeu->joueur[0].pos.x != (x-1) || configuration_jeu->joueur[0].pos.y != y) // on peut liberer des gens
			{
				if((rand() % 100) < 20) // 20% de chance de liberer les prisonniers
				{
					for(i = 0; i < configuration_jeu->nb_figurant_pieton; i++)
					{
						if(pieton[i].bloquer == 1 && pieton[i].pos.x == (x-1) && pieton[i].pos.y == y )
						{
							pieton[i].bloquer = 0;
						}
					}
				}
			}
		}
	}
	
	if((x + 1)  <  configuration_jeu->taille_carte_x)
	{
		int pos = ((x+1) * configuration_jeu->taille_carte_y) + y;
		if(carte[pos] == '#')
		{
			if(configuration_jeu->joueur[0].pos.x != (x+1) || configuration_jeu->joueur[0].pos.y != y) // on peut liberer des gens
			{
				if((rand() % 100) < 20) // 20% de chance de liberer les prisonniers
				{
					for(i = 0; i < configuration_jeu->nb_figurant_pieton; i++)
					{
						if(pieton[i].bloquer == 1 && pieton[i].pos.x == (x+1) && pieton[i].pos.y == y )
						{
							pieton[i].bloquer = 0;
						}
					}
				}
			}
		}
	}
	
	if((y + 1)  <  configuration_jeu->taille_carte_y)
	{
		int pos = (x * configuration_jeu->taille_carte_y) + y + 1;
		if(carte[pos] == '#')
		{
			if(configuration_jeu->joueur[0].pos.x != x || configuration_jeu->joueur[0].pos.y != (y+1)) // on peut liberer des gens
			{
				if((rand() % 100) < 20) // 20% de chance de liberer les prisonniers
				{
					for(i = 0; i < configuration_jeu->nb_figurant_pieton; i++)
					{
						if(pieton[i].bloquer == 1 && pieton[i].pos.x == x && pieton[i].pos.y == (y + 1))
						{
							pieton[i].bloquer = 0;
						}
					}
				}
			}
		}
	}
	
	if((y - 1)  >= 0)
	{
		int pos = (x * configuration_jeu->taille_carte_y) + y - 1;
		if(carte[pos] == '#')
		{
			if(configuration_jeu->joueur[0].pos.x != x || configuration_jeu->joueur[0].pos.y != (y - 1)) // on peut liberer des gens
			{
				if((rand() % 100) < 20) // 20% de chance de liberer les prisonniers
				{
					for(i = 0; i < configuration_jeu->nb_figurant_pieton; i++)
					{
						if(pieton[i].bloquer == 1 && pieton[i].pos.x == x && pieton[i].pos.y == (y - 1))
						{
							pieton[i].bloquer = 0;
						}
					}
				}
			}
		}
	}     
}

