#define _SVID_SOURCE
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
#include <sys/shm.h>
#include <sys/sem.h>
#include <errno.h>
#include <signal.h>
#include <time.h>
#include <string.h>
#include <netinet/in.h>
#include "definition.h"
#include "main.h"




/** 
* Permet de gerer les signaux en rapport avec le logger
* @param num_signal Numero du signal recu 
*/
void gestionnaire_signaux(int num_signal)
{
	if(num_signal == SIGQUIT || num_signal == SIGINT ||num_signal == SIGSEGV)
	{
		
		// on declenche la destruction de tous les outils ipc 
		if(conf_outil_IPC != NULL)
		{
			msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
			shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
			semctl(conf_outil_IPC->tab_semaphore_id,0,IPC_RMID,NULL);
		}
		exit(0);
	}
}


/**
* Ici on creer l'ensemble des outils ipc 
*  - creation segment memoire
*  - creation file de message 
*  - creation du tableau de semaphore en exclusion mutuelle pour la carte
* attend que des demande de configuration memoire 
* @param argc 
* @param argv[] 
* @return EXIT_SUCCESS si tout c'est bien passe sinon EXIT_FAILURE
*/
int main(int argc, char* argv[])
{
	srand(time(NULL));
	if(argc != 5)
	{
		printf("./logger grille.bin cleMsg cleSeg cleSem\n");
		exit(EXIT_FAILURE);
	}
	
	conf_outil_IPC = malloc(sizeof(config_outil_IPC_t));
	int i;
	void *memoire_partager;
	
	// definition du signal
	struct sigaction ordre;
	sigemptyset(&(ordre.sa_mask));
	
	ordre.sa_handler = gestionnaire_signaux;
	ordre.sa_flags = 0;
	
	sigaction(SIGQUIT,&ordre,NULL);
	sigaction(SIGINT,&ordre,NULL);
	sigaction(SIGSEGV,&ordre,NULL);
	
	conf_outil_IPC->clefFM = atoi(argv[2]);
	conf_outil_IPC->clefMP = atoi(argv[3]);
	conf_outil_IPC->clefTS = atoi(argv[4]);
	
	
	//creation de la file de message
	if((conf_outil_IPC->file_message_id = msgget(conf_outil_IPC->clefFM, S_IRUSR | S_IWUSR|  IPC_CREAT | IPC_EXCL)) == -1)
	{
		if(errno == EEXIST)
		{
			printf("La clef IPC est deja utiliser pour la file de message\n");
		}
		else
		{
			printf("Une erreur est survenue lors de la creation de la file de message\n");
		}
		exit(EXIT_FAILURE);
	}
	
	
	int taille[2];
	int file2 = open(argv[1], O_RDONLY);
	if(file2 == -1)
	{
		fprintf(stderr, "L’erreur suivante a été rencontrée lors de l’ouverture de ‘%s’ : ‘%s’.\n", argv[1], strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	if(read(file2, &taille, 2* sizeof(int)) == -1)
	{
		fprintf(stderr, "Erreur lors de la lecture de la carte %s: %s.\n", argv[1], strerror(errno));
		exit(EXIT_FAILURE);
	}
	close(file2);
	
	int nbfigurant = 8;
	//configuration du segment de memoire 
	
	if((conf_outil_IPC->seg_memoire_id = shmget(conf_outil_IPC->clefMP, sizeof(config_memoire)+(taille[0]*taille[1]*sizeof(char))+ nbfigurant*2*sizeof(Figurant_t) ,S_IRUSR | S_IWUSR|  IPC_CREAT | IPC_EXCL)) == -1)
	{
		if(errno == EEXIST)
		{
			printf("La clef IPC est deja utiliser pour la memoire partagee\n");
		}
		else
		{
			printf("Une erreur est survenue lors de la creation de la memoire partagee\n");
			msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		}
		exit(EXIT_FAILURE);
	}
	
	//attachement du segment de memoire partagee
	if((memoire_partager = shmat(conf_outil_IPC->seg_memoire_id, NULL,0)) == (void *)-1)
	{
		printf("Une erreur est survenue lors de l'attachement de la memoire partagee\n");
		msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
		exit(EXIT_FAILURE);
	}
	
	//creation d'un pointeur vers la memoire partagee et mise a 0 de toute la memoire
	config_memoire * configuration = (config_memoire *) memoire_partager;
	memset(memoire_partager,0,sizeof(config_memoire));
	
	configuration->argent_restant = 1000;// argent au depart
	configuration->nb_figurant_pieton = nbfigurant;
	configuration->nb_figurant_fourgon = nbfigurant;
	
	//on charge la carte dans la memoire
	lectureCarte(memoire_partager, argv[1]);
	
	printf("taille carte x %d y %d\n",configuration->taille_carte_x,configuration->taille_carte_y);
	
	int nb_sem = (configuration->taille_carte_x/4) * (configuration->taille_carte_y / 4);
	
	printf("nb_sem :%d\n",nb_sem);

	//configuration du tableau de semaphore
	if((conf_outil_IPC->tab_semaphore_id = semget(conf_outil_IPC->clefTS, nb_sem, S_IRUSR | S_IWUSR|  IPC_CREAT | IPC_EXCL)) == -1)
	{
		if(errno == EEXIST)
		{
			printf("La clef IPC est deja utiliser pour le tableau de semaphore\n");
		}
		else
		{
			printf("Une erreur est survenue lors de la creation de l'array de semaphore\n");
			msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
			shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
		}
		exit(EXIT_FAILURE);
	}
	
	//initialisation des valeurs des semaphores a 1 exclusion mutuelle pour chaque zone
	unsigned short *val = malloc(sizeof(unsigned short)*nb_sem);
	for(i = 0; i < nb_sem ; i++)
	{
		val[i] = 1;
	}
	
	if(semctl(conf_outil_IPC->tab_semaphore_id, 0, SETALL, val) == -1) 
	{
		perror("Erreur lors de l'initialisation des semaphores ");
		msgctl(conf_outil_IPC->file_message_id,IPC_RMID,NULL);
		shmctl(conf_outil_IPC->seg_memoire_id,IPC_RMID,NULL);
		semctl(conf_outil_IPC->tab_semaphore_id,0,IPC_RMID,NULL);
		exit(EXIT_FAILURE);
	}
	
	free(val);
	

	creeThread(nbfigurant, nbfigurant,memoire_partager);
	printf("creation des thread termner\n");
	
	while(1)
	{
		sleep(1);
		DemandeConfig();
	}	
}

/** 
* Fonction qui permet de lire la carte dans un fichier et de la charger
* en memoire correctement
* Argument : 
* void* memoire : pointeur memoire
* const char * nom_carte : nom du fichier de carte
* @param memoire Pointeur vers le segment memoire
* @param nom_carte Nom du fichier qui contient la carte a charger
*/
void lectureCarte(void * memoire, const char* nom_carte)
{
	
	config_memoire * configuration = (config_memoire *) memoire;
	
	
	char *emplacement_carte  = memoire + sizeof(config_memoire);
	
	int taille[2];
	
	int file = open(nom_carte, O_RDONLY);
	if(file == -1)
	{
		fprintf(stderr, "L’erreur suivante a ete rencontree lors de l’ouverture de ‘%s’ : ‘%s’.\n", nom_carte, strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	if(read(file, &taille, 2* sizeof(int)) == -1)
	{
		fprintf(stderr, "Erreur lors de la lecture de la carte %s: %s.\n", nom_carte, strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	configuration->taille_carte_x = taille[1];
	configuration->taille_carte_y = taille[0];
	
	if(read(file, emplacement_carte, taille[0] * taille[1] * sizeof(char)) == -1)
	{
		fprintf(stderr, "Erreur lors de la lecture de la carte %s: %s.\n", nom_carte, strerror(errno));
		exit(EXIT_FAILURE);
	}
}

/**
* Fonction qui retourne le nombre de figurant qu'il y a dans la carte
* @param carte Pointeur au debut de la carte dans le segment memoire
* @return Nombre de Figurant total
*/
int getNbFigurant(const char * carte)
{
	int taille[2];
	int i,nbcaseVide = 0;
	int file2 = open(carte, O_RDONLY);
	if(file2 == -1)
	{
		fprintf(stderr, "L’erreur suivante a été rencontrée lors de l’ouverture de ‘%s’ : ‘%s’.\n", carte, strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	if(read(file2, &taille, 2* sizeof(int)) == -1)
	{
		fprintf(stderr, "Erreur lors de la lecture de la carte %s: %s.\n", carte, strerror(errno));
		exit(EXIT_FAILURE);
	}
	
	char * varcarte = malloc(sizeof(char)*taille[0]*taille[1]);
	if(read(file2, varcarte, sizeof(char)*taille[0]*taille[1]) == -1)
	{
		fprintf(stderr, "Erreur lors de la lecture de la carte %s: %s.\n", carte, strerror(errno));
		exit(EXIT_FAILURE);
	}
	close(file2);
	
	for(i = 0; i < taille[0]*taille[1]; i++)
	{
		if(varcarte[i] == ' ')
		{
			nbcaseVide++;
		}
	}
	
	free(varcarte);
	return nbcaseVide/8;
}


/** 
* Fonction qui permet de lire dans la file de message chaque demande de configuration (type = 1)
* envoie la config memoire au demandeur type = idClient avec un message dans la file de message
*/
void DemandeConfig()
{
	// type 1 : demande config
	demande_config_t deplacement;
	if(msgrcv(conf_outil_IPC->file_message_id, &deplacement,sizeof(demande_config_t)-sizeof(long),1,IPC_NOWAIT) == -1)
	{
		if(errno == ENOMSG)
		{
			return;
		}
		else
		{
			fprintf(stderr, "Erreur lors de la reception d'une demande de config.\n");
		}
	}
	// type  = pid
	config_outil_IPC_t var = *conf_outil_IPC;
	var.type = deplacement.pid;
	
	if(msgsnd(conf_outil_IPC->file_message_id, &var,sizeof(config_outil_IPC_t)-sizeof(long),0) == -1)
	{
		fprintf(stderr, "Erreur lors de l'envoie de la config a quelqu'un.\n");
	}
}
